@@ -641,12 +641,12 @@ static void loadOutline(miniexp_t outlineExp, int offset, QStandardItem* parent,
                 }
 
                 QStandardItem* item = new QStandardItem(title);
-                item->setFlags(Qt::ItemIsEnabled);
+                item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
 
                 item->setData(destinationPage, Qt::UserRole + 1);
 
-                QStandardItem* pageItem = new QStandardItem(QString::number(destinationPage));
-                pageItem->setFlags(Qt::NoItemFlags);
+                QStandardItem* pageItem = item->clone();
+                pageItem->setText(QString::number(destinationPage));
                 pageItem->setTextAlignment(Qt::AlignRight);
 
                 parent->appendRow(QList< QStandardItem* >() << item << pageItem);
