@@ -1,6 +1,7 @@
 /*
 
 Copyright 2012-2013 Adam Reichold
+Copyright 2013 Thomas Etter
 
 This file is part of qpdfview.
 
@@ -32,7 +33,6 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <qmath.h>
 #include <QMenu>
 #include <QMessageBox>
-#include <QPrinter>
 #include <QProcess>
 #include <QProgressDialog>
 #include <QScrollBar>
@@ -60,6 +60,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include "presentationview.h"
 #include "searchtask.h"
 #include "miscellaneous.h"
+#include "documentlayout.h"
 
 Settings* DocumentView::s_settings = 0;
 ShortcutHandler* DocumentView::s_shortcutHandler = 0;
@@ -72,10 +73,11 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_pages(),
     m_filePath(),
     m_currentPage(-1),
+    m_wasModified(false),
     m_past(),
     m_future(),
+    m_layout(new SinglePageLayout),
     m_continuousMode(false),
-    m_layoutMode(SinglePageMode),
     m_scaleMode(ScaleFactorMode),
     m_scaleFactor(1.0),
     m_rotation(RotateBy0),
@@ -86,6 +88,7 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_thumbnailItems(),
     m_heightToIndex(),
     m_highlight(0),
+    m_thumbnailsOrientation(Qt::Vertical),
     m_thumbnailsScene(0),
     m_outlineModel(0),
     m_propertiesModel(0),
@@ -106,7 +109,6 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     setScene(new QGraphicsScene(this));
 
     setAcceptDrops(false);
-    setContextMenuPolicy(Qt::CustomContextMenu);
     setDragMode(QGraphicsView::ScrollHandDrag);
     setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
 
@@ -188,6 +190,11 @@ int DocumentView::currentPage() const
     return m_currentPage;
 }
 
+bool DocumentView::wasModified() const
+{
+    return m_wasModified;
+}
+
 QStringList DocumentView::openFilter()
 {
     QStringList openFilter;
@@ -253,18 +260,25 @@ void DocumentView::setContinousMode(bool continousMode)
 
 LayoutMode DocumentView::layoutMode() const
 {
-    return m_layoutMode;
+    return m_layout->layoutMode();
 }
 
 void DocumentView::setLayoutMode(LayoutMode layoutMode)
 {
-    if(m_layoutMode != layoutMode && layoutMode >= 0 && layoutMode < NumberOfLayoutModes)
+    if(m_layout->layoutMode() != layoutMode && layoutMode >= 0 && layoutMode < NumberOfLayoutModes)
     {
-        m_layoutMode = layoutMode;
-
-        if(m_currentPage != currentPageForPage(m_currentPage))
+        switch(layoutMode)
         {
-            m_currentPage = currentPageForPage(m_currentPage);
+        default:
+        case SinglePageMode: m_layout.reset(new SinglePageLayout); break;
+        case TwoPagesMode: m_layout.reset(new TwoPagesLayout); break;
+        case TwoPagesWithCoverPageMode: m_layout.reset(new TwoPagesWithCoverPageLayout); break;
+        case MultiplePagesMode: m_layout.reset(new MultiplePagesLayout); break;
+        }
+
+        if(m_currentPage != m_layout->currentPage(m_currentPage))
+        {
+            m_currentPage = m_layout->currentPage(m_currentPage);
 
             emit currentPageChanged(m_currentPage);
         }
@@ -272,7 +286,7 @@ void DocumentView::setLayoutMode(LayoutMode layoutMode)
         prepareScene();
         prepareView();
 
-        emit layoutModeChanged(m_layoutMode);
+        emit layoutModeChanged(layoutMode);
     }
 }
 
@@ -421,6 +435,21 @@ int DocumentView::searchProgress() const
     return m_searchTask->progress();
 }
 
+Qt::Orientation DocumentView::thumbnailsOrientation() const
+{
+    return m_thumbnailsOrientation;
+}
+
+void DocumentView::setThumbnailsOrientation(Qt::Orientation thumbnailsOrientation)
+{
+    if(m_thumbnailsOrientation != thumbnailsOrientation)
+    {
+        m_thumbnailsOrientation = thumbnailsOrientation;
+
+        prepareThumbnailsScene();
+    }
+}
+
 QStandardItemModel* DocumentView::outlineModel() const
 {
     return m_outlineModel;
@@ -477,6 +506,8 @@ bool DocumentView::open(const QString& filePath)
         m_filePath = filePath;
         m_currentPage = 1;
 
+        m_wasModified = false;
+
         m_past.clear();
         m_future.clear();
 
@@ -487,6 +518,8 @@ bool DocumentView::open(const QString& filePath)
 
         prepareThumbnailsScene();
 
+        emit documentChanged();
+
         emit filePathChanged(m_filePath);
         emit numberOfPagesChanged(m_pages.count());
         emit currentPageChanged(m_currentPage);
@@ -519,6 +552,8 @@ bool DocumentView::refresh()
 
         m_currentPage = qMin(m_currentPage, document->numberOfPages());
 
+        m_wasModified = false;
+
         prepareDocument(document);
 
         prepareScene();
@@ -526,6 +561,8 @@ bool DocumentView::refresh()
 
         prepareThumbnailsScene();
 
+        emit documentChanged();
+
         emit numberOfPagesChanged(m_pages.count());
         emit currentPageChanged(m_currentPage);
     }
@@ -564,6 +601,11 @@ bool DocumentView::save(const QString& filePath, bool withChanges)
                         }
                     }
 
+                    if(withChanges)
+                    {
+                        m_wasModified = false;
+                    }
+
                     delete[] data;
                     return true;
                 }
@@ -593,50 +635,12 @@ bool DocumentView::print(QPrinter* printer, const PrintOptions& printOptions)
 
 void DocumentView::previousPage()
 {
-    int previousPage = m_currentPage;
-
-    switch(m_layoutMode)
-    {
-    default:
-    case SinglePageMode:
-        previousPage -= 1;
-        break;
-    case TwoPagesMode:
-    case TwoPagesWithCoverPageMode:
-        previousPage -= 2;
-        break;
-    case MultiplePagesMode:
-        previousPage -= s_settings->documentView().pagesPerRow();
-        break;
-    }
-
-    previousPage = previousPage >= 1 ? previousPage : 1;
-
-    jumpToPage(previousPage);
+    jumpToPage(m_layout->previousPage(m_currentPage));
 }
 
 void DocumentView::nextPage()
 {
-    int nextPage = m_currentPage;
-
-    switch(m_layoutMode)
-    {
-    default:
-    case SinglePageMode:
-        nextPage += 1;
-        break;
-    case TwoPagesMode:
-    case TwoPagesWithCoverPageMode:
-        nextPage += 2;
-        break;
-    case MultiplePagesMode:
-        nextPage += s_settings->documentView().pagesPerRow();
-        break;
-    }
-
-    nextPage = qMin(nextPage, m_pages.count());
-
-    jumpToPage(nextPage);
+    jumpToPage(m_layout->nextPage(m_currentPage, m_pages.count()));
 }
 
 void DocumentView::firstPage()
@@ -656,7 +660,7 @@ void DocumentView::jumpToPage(int page, bool trackChange, qreal changeLeft, qrea
         qreal left = 0.0, top = 0.0;
         saveLeftAndTop(left, top);
 
-        if(m_currentPage != currentPageForPage(page) || qAbs(left - changeLeft) > 0.01 || qAbs(top - changeTop) > 0.01)
+        if(m_currentPage != m_layout->currentPage(page) || qAbs(left - changeLeft) > 0.01 || qAbs(top - changeTop) > 0.01)
         {
             if(trackChange)
             {
@@ -666,7 +670,7 @@ void DocumentView::jumpToPage(int page, bool trackChange, qreal changeLeft, qrea
                 emit canJumpChanged(true, false);
             }
 
-            m_currentPage = currentPageForPage(page);
+            m_currentPage = m_layout->currentPage(page);
 
             prepareView(changeLeft, changeTop);
 
@@ -761,23 +765,23 @@ void DocumentView::findPrevious()
 {
     if(m_currentResult != m_results.end())
     {
-        if(leftIndexForIndex(m_currentResult.key()) == m_currentPage - 1)
+        if(m_layout->leftIndex(m_currentResult.key()) == m_currentPage - 1)
         {
-            --m_currentResult;
+            m_currentResult = previousResult(m_currentResult);
         }
         else
         {
-            m_currentResult = --m_results.upperBound(m_currentPage - 1);
+            m_currentResult = previousResult(m_results.upperBound(m_currentPage - 1));
         }
     }
     else
     {
-        m_currentResult = --m_results.upperBound(m_currentPage - 1);
+        m_currentResult = previousResult(m_results.upperBound(m_currentPage - 1));
     }
 
     if(m_currentResult == m_results.end())
     {
-        m_currentResult = --m_results.end();
+        m_currentResult = previousResult(m_results.end());
     }
 
     if(m_currentResult != m_results.end())
@@ -796,7 +800,7 @@ void DocumentView::findNext()
 {
     if(m_currentResult != m_results.end())
     {
-        if(leftIndexForIndex(m_currentResult.key()) == m_currentPage - 1)
+        if(m_layout->leftIndex(m_currentResult.key()) == m_currentPage - 1)
         {
             ++m_currentResult;
         }
@@ -966,30 +970,9 @@ void DocumentView::on_verticalScrollBar_valueChanged(int value)
 
 void DocumentView::on_prefetch_timeout()
 {
-    int fromPage = m_currentPage, toPage = m_currentPage;
+    const QPair< int, int > prefetchRange = m_layout->prefetchRange(m_currentPage, m_pages.count());
 
-    switch(m_layoutMode)
-    {
-    default:
-    case SinglePageMode:
-        fromPage -= s_settings->documentView().prefetchDistance() / 2;
-        toPage += s_settings->documentView().prefetchDistance();
-        break;
-    case TwoPagesMode:
-    case TwoPagesWithCoverPageMode:
-        fromPage -= s_settings->documentView().prefetchDistance();
-        toPage += 2 * s_settings->documentView().prefetchDistance() + 1;
-        break;
-    case MultiplePagesMode:
-        fromPage -= s_settings->documentView().pagesPerRow() * (s_settings->documentView().prefetchDistance() / 2);
-        toPage += s_settings->documentView().pagesPerRow() * (s_settings->documentView().prefetchDistance() + 1) - 1;
-        break;
-    }
-
-    fromPage = qMax(fromPage, 1);
-    toPage = qMin(toPage, m_pages.count());
-
-    for(int index = fromPage - 1; index <= toPage - 1; ++index)
+    for(int index = prefetchRange.first - 1; index <= prefetchRange.second - 1; ++index)
     {
         m_pageItems.at(index)->startRender(true);
     }
@@ -1047,7 +1030,14 @@ void DocumentView::on_pages_linkClicked(const QString& url)
 {
     if(s_settings->documentView().openUrl())
     {
-        QDesktopServices::openUrl(QUrl(url));
+        QUrl resolvedUrl(url);
+
+        if(resolvedUrl.isRelative() && QFileInfo(url).isRelative())
+        {
+            resolvedUrl.setPath(QFileInfo(m_filePath).dir().filePath(url));
+        }
+
+        QDesktopServices::openUrl(resolvedUrl);
     }
     else
     {
@@ -1114,6 +1104,11 @@ void DocumentView::on_pages_sourceRequested(int page, const QPointF& pos)
 #endif // WITH_SYNCTEX
 }
 
+void DocumentView::on_pages_wasModified()
+{
+    m_wasModified = true;
+}
+
 void DocumentView::resizeEvent(QResizeEvent* event)
 {
     QGraphicsView::resizeEvent(event);
@@ -1130,55 +1125,33 @@ void DocumentView::resizeEvent(QResizeEvent* event)
 
 void DocumentView::keyPressEvent(QKeyEvent* event)
 {
-    QKeySequence keySequence(event->modifiers() + event->key());
+    const QKeySequence keySequence(event->modifiers() + event->key());
 
-    if(!m_continuousMode)
-    {
-        if(s_shortcutHandler->matchesSkipBackward(keySequence) && verticalScrollBar()->value() == verticalScrollBar()->minimum() && m_currentPage != 1)
-        {
-            previousPage();
-
-            verticalScrollBar()->setValue(verticalScrollBar()->maximum());
-
-            event->accept();
-            return;
-        }
-        else if(s_shortcutHandler->matchesSkipForward(keySequence) && verticalScrollBar()->value() == verticalScrollBar()->maximum() && m_currentPage != currentPageForPage(m_pages.count()))
-        {
-            nextPage();
-
-            verticalScrollBar()->setValue(verticalScrollBar()->minimum());
-
-            event->accept();
-            return;
-        }
-    }
-
-    int key = -1;
+    int maskedKey = -1;
 
     if(s_shortcutHandler->matchesSkipBackward(keySequence))
     {
-        key = Qt::Key_PageUp;
+        maskedKey = Qt::Key_PageUp;
     }
     else if(s_shortcutHandler->matchesSkipForward(keySequence))
     {
-        key = Qt::Key_PageDown;
+        maskedKey = Qt::Key_PageDown;
     }
     else if(s_shortcutHandler->matchesMoveUp(keySequence))
     {
-        key = Qt::Key_Up;
+        maskedKey = Qt::Key_Up;
     }
     else if(s_shortcutHandler->matchesMoveDown(keySequence))
     {
-        key = Qt::Key_Down;
+        maskedKey = Qt::Key_Down;
     }
     else if(s_shortcutHandler->matchesMoveLeft(keySequence))
     {
-        key = Qt::Key_Left;
+        maskedKey = Qt::Key_Left;
     }
     else if(s_shortcutHandler->matchesMoveRight(keySequence))
     {
-        key = Qt::Key_Right;
+        maskedKey = Qt::Key_Right;
     }
     else if(event->key() == Qt::Key_PageUp || event->key() == Qt::Key_PageDown ||
             event->key() == Qt::Key_Up || event->key() == Qt::Key_Down ||
@@ -1188,16 +1161,67 @@ void DocumentView::keyPressEvent(QKeyEvent* event)
         return;
     }
 
-    if(key != -1)
+    if(maskedKey != -1)
     {
-        QKeyEvent keyEvent(event->type(), key, Qt::NoModifier, event->text(), event->isAutoRepeat(), event->count());
+        QKeyEvent keyEvent(event->type(), maskedKey, Qt::NoModifier, event->text(), event->isAutoRepeat(), event->count());
         QGraphicsView::keyPressEvent(&keyEvent);
-
-        event->accept();
-        return;
+    }
+    else
+    {
+        QGraphicsView::keyPressEvent(event);
     }
 
-    QGraphicsView::keyPressEvent(event);
+    if(maskedKey == Qt::Key_PageUp || maskedKey == Qt::Key_PageDown ||
+       maskedKey == Qt::Key_Up || maskedKey == Qt::Key_Down ||
+       maskedKey == Qt::Key_Left || maskedKey == Qt::Key_Right)
+    {
+        foreach(const PageItem* page, m_pageItems)
+        {
+            if(page->showsAnnotationOverlay() || page->showsFormFieldOverlay())
+            {
+                return;
+            }
+        }
+
+        if(!m_continuousMode)
+        {
+            if(maskedKey == Qt::Key_PageUp && verticalScrollBar()->value() == verticalScrollBar()->minimum() && m_currentPage != 1)
+            {
+                previousPage();
+
+                verticalScrollBar()->setValue(verticalScrollBar()->maximum());
+
+                event->accept();
+                return;
+            }
+            else if(maskedKey == Qt::Key_PageDown && verticalScrollBar()->value() == verticalScrollBar()->maximum() && m_currentPage != m_layout->currentPage(m_pages.count()))
+            {
+                nextPage();
+
+                verticalScrollBar()->setValue(verticalScrollBar()->minimum());
+
+                event->accept();
+                return;
+            }
+        }
+
+        if((maskedKey == Qt::Key_Up && verticalScrollBar()->minimum() == verticalScrollBar()->maximum()) ||
+           (maskedKey == Qt::Key_Left && !horizontalScrollBar()->isVisible()))
+        {
+            previousPage();
+
+            event->accept();
+            return;
+        }
+        else if((maskedKey == Qt::Key_Down && verticalScrollBar()->minimum() == verticalScrollBar()->maximum()) ||
+                (maskedKey == Qt::Key_Right && !horizontalScrollBar()->isVisible()))
+        {
+            nextPage();
+
+            event->accept();
+            return;
+        }
+    }
 }
 
 void DocumentView::wheelEvent(QWheelEvent* event)
@@ -1251,7 +1275,7 @@ void DocumentView::wheelEvent(QWheelEvent* event)
                 event->accept();
                 return;
             }
-            else if(event->delta() < 0 && verticalScrollBar()->value() == verticalScrollBar()->maximum() && m_currentPage != currentPageForPage(m_pages.count()))
+            else if(event->delta() < 0 && verticalScrollBar()->value() == verticalScrollBar()->maximum() && m_currentPage != m_layout->currentPage(m_pages.count()))
             {
                 nextPage();
 
@@ -1484,27 +1508,30 @@ bool DocumentView::printUsingQt(QPrinter* printer, const PrintOptions& printOpti
 
         QApplication::processEvents();
 
+        painter.save();
+
         const Model::Page* page = m_pages.at(index);
 
-        const qreal pageWidth = printer->physicalDpiX() / 72.0 * page->size().width();
-        const qreal pageHeight = printer->physicalDpiY() / 72.0 * page->size().width();
-
-        qreal scaleFactorX = 1.0, scaleFactorY = 1.0;
-
         if(printOptions.fitToPage)
         {
-            scaleFactorX = scaleFactorY = qMin(printer->width() / pageWidth, printer->height() / pageHeight);
+            const qreal pageWidth = printer->physicalDpiX() / 72.0 * page->size().width();
+            const qreal pageHeight = printer->physicalDpiY() / 72.0 * page->size().width();
+
+            const qreal scaleFactor = qMin(printer->width() / pageWidth, printer->height() / pageHeight);
+
+            painter.setTransform(QTransform::fromScale(scaleFactor, scaleFactor));
         }
         else
         {
-            scaleFactorX = printer->logicalDpiX(); scaleFactorX /= printer->physicalDpiX();
-            scaleFactorY = printer->logicalDpiY(); scaleFactorY /= printer->physicalDpiY();
+            const qreal scaleFactorX = static_cast< qreal >(printer->logicalDpiX()) / static_cast< qreal >(printer->physicalDpiX());
+            const qreal scaleFactorY = static_cast< qreal >(printer->logicalDpiY()) / static_cast< qreal >(printer->physicalDpiY());
+
+            painter.setTransform(QTransform::fromScale(scaleFactorX, scaleFactorY));
         }
 
-        QImage image = page->render(printer->physicalDpiX(), printer->physicalDpiY());
+        painter.drawImage(QPointF(), page->render(printer->physicalDpiX(), printer->physicalDpiY()));
 
-        painter.setTransform(QTransform::fromScale(scaleFactorX, scaleFactorY));
-        painter.drawImage(QPointF(), image);
+        painter.restore();
 
         if(index < toPage - 1)
         {
@@ -1515,6 +1542,8 @@ bool DocumentView::printUsingQt(QPrinter* printer, const PrintOptions& printOpti
 
         if(progressDialog->wasCanceled())
         {
+            printer->abort();
+
             return false;
         }
     }
@@ -1522,80 +1551,6 @@ bool DocumentView::printUsingQt(QPrinter* printer, const PrintOptions& printOpti
     return true;
 }
 
-int DocumentView::currentPageForPage(int page) const
-{
-    int currentPage = -1;
-
-    switch(m_layoutMode)
-    {
-    default:
-    case SinglePageMode:
-        currentPage = page;
-        break;
-    case TwoPagesMode:
-        currentPage = page % 2 != 0 ? page : page - 1;
-        break;
-    case TwoPagesWithCoverPageMode:
-        currentPage = page == 1 ? page : (page % 2 == 0 ? page : page - 1);
-        break;
-    case MultiplePagesMode:
-        currentPage = page - ((page - 1) % s_settings->documentView().pagesPerRow());
-        break;
-    }
-
-    return currentPage;
-}
-
-int DocumentView::leftIndexForIndex(int index) const
-{
-    int leftIndex = -1;
-
-    switch(m_layoutMode)
-    {
-    default:
-    case SinglePageMode:
-        leftIndex = index;
-        break;
-    case TwoPagesMode:
-        leftIndex = index % 2 == 0 ? index : index - 1;
-        break;
-    case TwoPagesWithCoverPageMode:
-        leftIndex = index == 0 ? index : (index % 2 != 0 ? index : index - 1);
-        break;
-    case MultiplePagesMode:
-        leftIndex = index - (index % s_settings->documentView().pagesPerRow());
-        break;
-    }
-
-    return leftIndex;
-}
-
-int DocumentView::rightIndexForIndex(int index) const
-{
-    int rightIndex = -1;
-
-    switch(m_layoutMode)
-    {
-    default:
-    case SinglePageMode:
-        rightIndex = index;
-        break;
-    case TwoPagesMode:
-        rightIndex = index % 2 == 0 ? index + 1 : index;
-        break;
-    case TwoPagesWithCoverPageMode:
-        rightIndex = index % 2 != 0 ? index + 1 : index;
-        break;
-    case MultiplePagesMode:
-        rightIndex = index - (index % s_settings->documentView().pagesPerRow()) + s_settings->documentView().pagesPerRow() - 1;
-        break;
-    }
-
-    rightIndex = qMin(rightIndex, m_pages.count() - 1);
-
-    return rightIndex;
-}
-
 void DocumentView::saveLeftAndTop(qreal& left, qreal& top) const
 {
     const PageItem* page = m_pageItems.at(m_currentPage - 1);
@@ -1610,6 +1565,28 @@ void DocumentView::saveLeftAndTop(qreal& left, qreal& top) const
     top = top >= 0.0 ? top : 0.0;
 }
 
+void DocumentView::loadFallbackOutline()
+{
+    if(m_outlineModel->rowCount() > 0)
+    {
+        return;
+    }
+
+    for(int page = 1; page <= m_pages.count(); ++page)
+    {
+        QStandardItem* item = new QStandardItem(tr("Page %1").arg(page));
+        item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
+
+        item->setData(page, Qt::UserRole + 1);
+
+        QStandardItem* pageItem = item->clone();
+        pageItem->setText(QString::number(page));
+        pageItem->setTextAlignment(Qt::AlignRight);
+
+        m_outlineModel->appendRow(QList< QStandardItem* >() << item << pageItem);
+    }
+}
+
 void DocumentView::prepareDocument(Model::Document* document)
 {
     m_prefetchTimer->blockSignals(true);
@@ -1655,6 +1632,8 @@ void DocumentView::prepareDocument(Model::Document* document)
     m_document->loadOutline(m_outlineModel);
     m_document->loadProperties(m_propertiesModel);
 
+    loadFallbackOutline();
+
     if(s_settings->documentView().prefetch())
     {
         m_prefetchTimer->blockSignals(false);
@@ -1671,7 +1650,6 @@ void DocumentView::preparePages()
     {
         PageItem* page = new PageItem(m_pages.at(index), index);
 
-        page->setPhysicalDpi(physicalDpiX(), physicalDpiY());
         page->setInvertColors(m_invertColors);
         page->setRubberBandMode(m_rubberBandMode);
 
@@ -1685,6 +1663,8 @@ void DocumentView::preparePages()
         connect(page, SIGNAL(rubberBandFinished()), SLOT(on_pages_rubberBandFinished()));
 
         connect(page, SIGNAL(sourceRequested(int,QPointF)), SLOT(on_pages_sourceRequested(int,QPointF)));
+
+        connect(page, SIGNAL(wasModified()), SLOT(on_pages_wasModified()));
     }
 }
 
@@ -1697,7 +1677,6 @@ void DocumentView::prepareThumbnails()
     {
         ThumbnailItem* page = new ThumbnailItem(m_pages.at(index), index);
 
-        page->setPhysicalDpi(physicalDpiX(), physicalDpiY());
         page->setInvertColors(m_invertColors);
 
         m_thumbnailsScene->addItem(page);
@@ -1731,74 +1710,51 @@ void DocumentView::prepareBackground()
 
 void DocumentView::prepareScene()
 {
-    const qreal pageSpacing = s_settings->documentView().pageSpacing();
-
     // prepare scale factor and rotation
 
-    for(int index = 0; index < m_pageItems.count(); ++index)
+    const qreal visibleWidth = m_layout->visibleWidth(viewport()->width());
+    const qreal visibleHeight = m_layout->visibleHeight(viewport()->height());
+
+    foreach(PageItem* page, m_pageItems)
     {
-        PageItem* page = m_pageItems.at(index);
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
 
-        if(m_scaleMode != ScaleFactorMode)
+        page->setDevicePixelRatio(devicePixelRatio());
+
+#endif // QT_VERSION
+
+        page->setResolution(logicalDpiX(), logicalDpiY());
+
+        qreal pageWidth = 0.0;
+        qreal pageHeight = 0.0;
+
+        switch(m_rotation)
         {
-            qreal visibleWidth = 0.0;
-            qreal visibleHeight = 0.0;
-
-            qreal pageWidth = 0.0;
-            qreal pageHeight = 0.0;
-
-            qreal scaleFactor = 1.0;
-
-            switch(m_layoutMode)
-            {
-            default:
-            case SinglePageMode:
-                visibleWidth = viewport()->width() - 6.0 - 2.0 * pageSpacing;
-                break;
-            case TwoPagesMode:
-            case TwoPagesWithCoverPageMode:
-                visibleWidth = (viewport()->width() - 6.0 - 3 * pageSpacing) / 2;
-                break;
-            case MultiplePagesMode:
-                visibleWidth = (viewport()->width() - 6.0 - (s_settings->documentView().pagesPerRow() + 1) * pageSpacing) / s_settings->documentView().pagesPerRow();
-                break;
-            }
-
-            visibleHeight = viewport()->height() - 2.0 * pageSpacing;
-
-            switch(m_rotation)
-            {
-            default:
-            case RotateBy0:
-            case RotateBy180:
-                pageWidth = physicalDpiX() / 72.0 * page->size().width();
-                pageHeight = physicalDpiY() / 72.0 * page->size().height();
-                break;
-            case RotateBy90:
-            case RotateBy270:
-                pageWidth = physicalDpiX() / 72.0 * page->size().height();
-                pageHeight = physicalDpiY() / 72.0 * page->size().width();
-                break;
-            }
-
-            switch(m_scaleMode)
-            {
-            default:
-            case ScaleFactorMode:
-                break;
-            case FitToPageWidthMode:
-                scaleFactor = visibleWidth / pageWidth;
-                break;
-            case FitToPageSizeMode:
-                scaleFactor = qMin(visibleWidth / pageWidth, visibleHeight / pageHeight);
-                break;
-            }
-
-            page->setScaleFactor(scaleFactor);
+        default:
+        case RotateBy0:
+        case RotateBy180:
+            pageWidth = logicalDpiX() / 72.0 * page->size().width();
+            pageHeight = logicalDpiY() / 72.0 * page->size().height();
+            break;
+        case RotateBy90:
+        case RotateBy270:
+            pageWidth = logicalDpiX() / 72.0 * page->size().height();
+            pageHeight = logicalDpiY() / 72.0 * page->size().width();
+            break;
         }
-        else
+
+        switch(m_scaleMode)
         {
+        default:
+        case ScaleFactorMode:
             page->setScaleFactor(m_scaleFactor);
+            break;
+        case FitToPageWidthMode:
+            page->setScaleFactor(visibleWidth / pageWidth);
+            break;
+        case FitToPageSizeMode:
+            page->setScaleFactor(qMin(visibleWidth / pageWidth, visibleHeight / pageHeight));
+            break;
         }
 
         page->setRotation(m_rotation);
@@ -1812,86 +1768,19 @@ void DocumentView::prepareScene()
 
     qreal left = 0.0;
     qreal right = 0.0;
-    qreal height = pageSpacing;
+    qreal height = s_settings->documentView().pageSpacing();
 
-    for(int index = 0; index < m_pageItems.count(); ++index)
-    {
-        PageItem* page = m_pageItems.at(index);
-        const QRectF boundingRect = page->boundingRect();
-
-        switch(m_layoutMode)
-        {
-        default:
-        case SinglePageMode:
-            page->setPos(-boundingRect.left() - 0.5 * boundingRect.width(), height - boundingRect.top());
-
-            m_heightToIndex.insert(-height + pageSpacing + 0.3 * pageHeight, index);
-
-            pageHeight = boundingRect.height();
-
-            left = qMin(left, -0.5f * boundingRect.width() - pageSpacing);
-            right = qMax(right, 0.5f * boundingRect.width() + pageSpacing);
-            height += pageHeight + pageSpacing;
-
-            break;
-        case TwoPagesMode:
-        case TwoPagesWithCoverPageMode:
-            if(index == leftIndexForIndex(index))
-            {
-                page->setPos(-boundingRect.left() - boundingRect.width() - 0.5 * pageSpacing, height - boundingRect.top());
-
-                m_heightToIndex.insert(-height + pageSpacing + 0.3 * pageHeight, index);
-
-                pageHeight = boundingRect.height();
-
-                left = qMin(left, -boundingRect.width() - 1.5f * pageSpacing);
-
-                if(index == rightIndexForIndex(index))
-                {
-                    right = qMax(right, 0.5f * pageSpacing);
-                    height += pageHeight + pageSpacing;
-                }
-            }
-            else
-            {
-                page->setPos(-boundingRect.left() + 0.5 * pageSpacing, height - boundingRect.top());
-
-                pageHeight = qMax(pageHeight, boundingRect.height());
-
-                right = qMax(right, boundingRect.width() + 1.5f * pageSpacing);
-                height += pageHeight + pageSpacing;
-            }
-
-            break;
-        case MultiplePagesMode:
-            page->setPos(left - boundingRect.left() + pageSpacing, height - boundingRect.top());
-
-            pageHeight = qMax(pageHeight, boundingRect.height());
-            left += boundingRect.width() + pageSpacing;
-
-            if(index == leftIndexForIndex(index))
-            {
-                m_heightToIndex.insert(-height + pageSpacing + 0.3 * pageHeight, index);
-            }
-
-            if(index == rightIndexForIndex(index))
-            {
-                height += pageHeight + pageSpacing;
-                pageHeight = 0.0;
-
-                right = qMax(right, left + pageSpacing);
-                left = 0.0;
-            }
-
-            break;
-        }
-    }
+    m_layout->prepareLayout(m_pageItems,
+                            m_heightToIndex, pageHeight,
+                            left, right, height);
 
     scene()->setSceneRect(left, 0.0, right - left, height);
 }
 
 void DocumentView::prepareView(qreal changeLeft, qreal changeTop)
 {
+    const bool highlightCurrentThumbnail = s_settings->documentView().highlightCurrentThumbnail();
+
     qreal left = scene()->sceneRect().left();
     qreal top = scene()->sceneRect().top();
     qreal width = scene()->sceneRect().width();
@@ -1900,8 +1789,6 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop)
     int horizontalValue = 0;
     int verticalValue = 0;
 
-    const bool highlightCurrentThumbnail = s_settings->documentView().highlightCurrentThumbnail();
-
     for(int index = 0; index < m_pageItems.count(); ++index)
     {
         PageItem* page = m_pageItems.at(index);
@@ -1919,7 +1806,7 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop)
         }
         else
         {
-            if(leftIndexForIndex(index) == m_currentPage - 1)
+            if(m_layout->leftIndex(index) == m_currentPage - 1)
             {
                 page->setVisible(true);
 
@@ -1967,8 +1854,9 @@ void DocumentView::prepareThumbnailsScene()
     const qreal thumbnailSpacing = s_settings->documentView().thumbnailSpacing();
 
     qreal left = 0.0;
-    qreal right = 0.0;
-    qreal height = thumbnailSpacing;
+    qreal right = m_thumbnailsOrientation == Qt::Vertical ? 0.0 : thumbnailSpacing;
+    qreal top = 0.0;
+    qreal bottom = m_thumbnailsOrientation == Qt::Vertical ? thumbnailSpacing : 0.0;
 
     const bool limitThumbnailsToResults = s_settings->documentView().limitThumbnailsToResults();
 
@@ -1989,8 +1877,16 @@ void DocumentView::prepareThumbnailsScene()
 
         // prepare scale factor
 
-        const qreal pageWidth = physicalDpiX() / 72.0 * page->size().width();
-        const qreal pageHeight = physicalDpiY() / 72.0 * page->size().height();
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+
+        page->setDevicePixelRatio(devicePixelRatio());
+
+#endif // QT_VERSION
+
+        page->setResolution(logicalDpiX(), logicalDpiY());
+
+        const qreal pageWidth = logicalDpiX() / 72.0 * page->size().width();
+        const qreal pageHeight = logicalDpiY() / 72.0 * page->size().height();
 
         page->setScaleFactor(qMin(s_settings->documentView().thumbnailSize() / pageWidth, s_settings->documentView().thumbnailSize() / pageHeight));
 
@@ -1998,14 +1894,25 @@ void DocumentView::prepareThumbnailsScene()
 
         const QRectF boundingRect = page->boundingRect();
 
-        page->setPos(-boundingRect.left() - 0.5 * boundingRect.width(), height - boundingRect.top());
+        if(m_thumbnailsOrientation == Qt::Vertical)
+        {
+            page->setPos(-boundingRect.left() - 0.5 * boundingRect.width(), bottom - boundingRect.top());
 
-        left = qMin(left, -0.5f * boundingRect.width() - thumbnailSpacing);
-        right = qMax(right, 0.5f * boundingRect.width() + thumbnailSpacing);
-        height += boundingRect.height() + thumbnailSpacing;
+            left = qMin(left, -0.5f * boundingRect.width() - thumbnailSpacing);
+            right = qMax(right, 0.5f * boundingRect.width() + thumbnailSpacing);
+            bottom += boundingRect.height() + thumbnailSpacing;
+        }
+        else
+        {
+            page->setPos(right - boundingRect.left(), -boundingRect.top() - 0.5 * boundingRect.height());
+
+            top = qMin(top, -0.5f * boundingRect.height() - thumbnailSpacing);
+            bottom = qMax(bottom, 0.5f * boundingRect.height() + thumbnailSpacing);
+            right += boundingRect.width() + thumbnailSpacing;
+        }
     }
 
-    m_thumbnailsScene->setSceneRect(left, 0.0, right - left, height);
+    m_thumbnailsScene->setSceneRect(left, top, right - left, bottom - top);
 }
 
 void DocumentView::prepareHighlight(int index, const QRectF& rect)
@@ -2028,3 +1935,8 @@ void DocumentView::prepareHighlight(int index, const QRectF& rect)
 
     viewport()->update();
 }
+
+DocumentView::Results::iterator DocumentView::previousResult(const Results::iterator& result)
+{
+    return result != m_results.begin() ? result - 1 : m_results.end();
+}
