@@ -23,6 +23,8 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef GLOBAL_H
 #define GLOBAL_H
 
+#include <QPair>
+
 enum Rotation
 {
     RotateBy0 = 0,
@@ -57,4 +59,7 @@ enum ScaleMode
     NumberOfScaleModes = 3
 };
 
+typedef QPair< int, QString > Jump;
+typedef QList< QPair< int, QString > > JumpList;
+
 #endif // GLOBAL_H
