@@ -26,12 +26,6 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QMainWindow>
 
-#ifdef WITH_SQL
-
-#include <QSqlDatabase>
-
-#endif // WITH_SQL
-
 #ifdef WITH_DBUS
 
 #include <QDBusAbstractAdaptor>
@@ -53,16 +47,17 @@ class TabWidget;
 class TreeView;
 class ComboBox;
 class SpinBox;
-class ProgressLineEdit;
+class SearchLineEdit;
 class RecentlyUsedMenu;
 class BookmarkMenu;
+class Database;
 
 class MainWindow : public QMainWindow
 {
     Q_OBJECT
     
 public:
-    MainWindow(const QString& instanceName = "", QWidget* parent = 0);
+    explicit MainWindow(QWidget* parent = 0);
 
     QSize sizeHint() const;
     QMenu* createPopupMenu();
@@ -78,6 +73,9 @@ public slots:
 protected slots:
     void on_tabWidget_currentChanged(int index);
     void on_tabWidget_tabCloseRequested(int index);
+    void on_tabWidget_tabContextMenuRequested(const QPoint& globalPos, int index);
+
+    void on_currentTab_documentChanged();
 
     void on_currentTab_filePathChanged(const QString& filePath);
     void on_currentTab_numberOfPagesChaned(int numberOfPages);
@@ -111,6 +109,7 @@ protected slots:
 
     void on_open_triggered();
     void on_openInNewTab_triggered();
+    void on_openContainingFolder_triggered();
     void on_refresh_triggered();
     void on_saveCopy_triggered();
     void on_saveAs_triggered();
@@ -129,8 +128,6 @@ protected slots:
     void on_jumpForward_triggered();
 
     void on_search_triggered();
-    void on_search_returnPressed(const Qt::KeyboardModifiers& modifiers);
-    void on_search_timeout();
     void on_findPrevious_triggered();
     void on_findNext_triggered();
     void on_cancelSearch_triggered();
@@ -183,14 +180,20 @@ protected slots:
     void on_contents_triggered();
     void on_about_triggered();
 
-    void on_highlightAll_clicked(bool checked);
+    void on_focusCurrentPage_activated();
+    void on_focusScaleFactor_activated();
 
-    void on_model_reset();
+    void on_searchInitiated(const QString& text, bool allTabs);
+    void on_highlightAll_clicked(bool checked);
 
     void on_outline_clicked(const QModelIndex& index);
 
+    void on_thumbnails_dockLocationChanged(Qt::DockWidgetArea area);
     void on_thumbnails_verticalScrollBar_valueChanged(int value);
 
+    void on_database_tabRestored(const QString& filePath, bool continousMode, LayoutMode layoutMode, ScaleMode scaleMode, qreal scaleFactor, Rotation rotation, int currentPage);
+    void on_database_bookmarkRestored(const QString& filePath, const JumpList& pages);
+
 protected:
     void closeEvent(QCloseEvent* event);
 
@@ -198,27 +201,35 @@ protected:
     void dropEvent(QDropEvent* event);
 
 private:
+    Q_DISABLE_COPY(MainWindow)
+
     static Settings* s_settings;
+    static Database* s_database;
 
     TabWidget* m_tabWidget;
 
     DocumentView* currentTab() const;
     DocumentView* tab(int index) const;
+    QList< DocumentView* > tabs() const;
 
     bool senderIsCurrentTab() const;
 
-    QString windowTitleSuffixForCurrentTab() const;
+    void setWindowTitleForCurrentTab();
 
     BookmarkMenu* bookmarkForCurrentTab() const;
 
+    void saveTabs() const;
+    void saveBookmarks() const;
+
+    bool saveModifications(DocumentView* tab);
+
     SpinBox* m_currentPageSpinBox;
     QWidgetAction* m_currentPageAction;
 
     ComboBox* m_scaleFactorComboBox;
     QWidgetAction* m_scaleFactorAction;
 
-    ProgressLineEdit* m_searchProgressLineEdit;
-    QTimer* m_searchTimer;
+    SearchLineEdit* m_searchLineEdit;
     QCheckBox* m_matchCaseCheckBox;
     QCheckBox* m_highlightAllCheckBox;
 
@@ -226,6 +237,7 @@ private:
 
     QAction* m_openAction;
     QAction* m_openInNewTabAction;
+    QAction* m_openContainingFolderAction;
     QAction* m_refreshAction;
     QAction* m_saveCopyAction;
     QAction* m_saveAsAction;
@@ -302,7 +314,8 @@ private:
     QToolBar* m_editToolBar;
     QToolBar* m_viewToolBar;
 
-    QToolBar* m_searchToolBar;
+    QShortcut* m_focusCurrentPageShortcut;
+    QShortcut* m_focusScaleFactorShortcut;
 
     QToolBar* createToolBar(const QString& text, const QString& objectName, const QStringList& actionNames, const QList< QAction* >& actions);
 
@@ -317,6 +330,9 @@ private:
     QDockWidget* m_thumbnailsDock;
     QGraphicsView* m_thumbnailsView;
 
+    QDockWidget* m_searchDock;
+    QWidget* m_searchWidget;
+
     QDockWidget* createDock(const QString& text, const QString& objectName, const QKeySequence& toggleViewShortcut);
 
     void createDocks();
@@ -331,23 +347,6 @@ private:
 
     void createMenus();
 
-#ifdef WITH_SQL
-
-    QSqlDatabase m_database;
-
-#endif // WITH_SQL
-
-    void createDatabase();
-
-    void restoreTabs();
-    void saveTabs();
-
-    void restoreBookmarks();
-    void saveBookmarks();
-
-    void restorePerFileSettings(DocumentView* tab);
-    void savePerFileSettings(const DocumentView* tab);
-
 };
 
 #ifdef WITH_DBUS
