@@ -23,6 +23,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QMenu>
 #include <QMouseEvent>
+#include <QTimer>
 
 GraphicsCompositionModeEffect::GraphicsCompositionModeEffect(QPainter::CompositionMode compositionMode, QObject* parent) : QGraphicsEffect(parent),
     m_compositionMode(compositionMode)
@@ -34,7 +35,6 @@ void GraphicsCompositionModeEffect::draw(QPainter* painter)
     painter->save();
 
     painter->setCompositionMode(m_compositionMode);
-
     drawSource(painter);
 
     painter->restore();
@@ -43,6 +43,7 @@ void GraphicsCompositionModeEffect::draw(QPainter* painter)
 
 TabBar::TabBar(QWidget* parent) : QTabBar(parent)
 {
+    setContextMenuPolicy(Qt::CustomContextMenu);
 }
 
 void TabBar::mousePressEvent(QMouseEvent* event)
@@ -59,6 +60,8 @@ TabWidget::TabWidget(QWidget* parent) : QTabWidget(parent),
     m_tabBarPolicy(TabBarAsNeeded)
 {
     setTabBar(new TabBar(this));
+
+    connect(tabBar(), SIGNAL(customContextMenuRequested(QPoint)), SLOT(on_tabBar_customContextMenuRequested(QPoint)));
 }
 
 TabWidget::TabBarPolicy TabWidget::tabBarPolicy() const
@@ -84,6 +87,11 @@ void TabWidget::setTabBarPolicy(TabWidget::TabBarPolicy tabBarPolicy)
     }
 }
 
+void TabWidget::on_tabBar_customContextMenuRequested(const QPoint &pos)
+{
+    emit tabContextMenuRequested(tabBar()->mapToGlobal(pos), tabBar()->tabAt(pos));
+}
+
 void TabWidget::tabInserted(int index)
 {
     QTabWidget::tabInserted(index);
@@ -104,8 +112,19 @@ void TabWidget::tabRemoved(int index)
     }
 }
 
-TreeView::TreeView(QWidget* parent) : QTreeView(parent)
+TreeView::TreeView(int expansionRole, QWidget* parent) : QTreeView(parent),
+    m_expansionRole(expansionRole)
 {
+    connect(this, SIGNAL(expanded(QModelIndex)), SLOT(on_expanded(QModelIndex)));
+    connect(this, SIGNAL(collapsed(QModelIndex)), SLOT(on_collapsed(QModelIndex)));
+}
+
+void TreeView::expandAbove(const QModelIndex& child)
+{
+    for(QModelIndex index = child.parent(); index.isValid(); index = index.parent())
+    {
+        expand(index);
+    }
 }
 
 void TreeView::expandAll(const QModelIndex& index)
@@ -142,6 +161,26 @@ void TreeView::collapseAll(const QModelIndex& index)
     }
 }
 
+void TreeView::restoreExpansion(const QModelIndex& index)
+{
+    if(index.isValid())
+    {
+        if(model()->data(index, m_expansionRole).toBool())
+        {
+            expand(index);
+        }
+        else
+        {
+            collapse(index);
+        }
+    }
+
+    for(int row = 0; row < model()->rowCount(index); ++row)
+    {
+        restoreExpansion(model()->index(row, 0, index));
+    }
+}
+
 void TreeView::contextMenuEvent(QContextMenuEvent* event)
 {
     QTreeView::contextMenuEvent(event);
@@ -166,6 +205,16 @@ void TreeView::contextMenuEvent(QContextMenuEvent* event)
     }
 }
 
+void TreeView::on_expanded(const QModelIndex& index)
+{
+    model()->setData(index, true, m_expansionRole);
+}
+
+void TreeView::on_collapsed(const QModelIndex& index)
+{
+    model()->setData(index, false, m_expansionRole);
+}
+
 LineEdit::LineEdit(QWidget* parent) : QLineEdit(parent)
 {
 }
@@ -224,8 +273,7 @@ void ProgressLineEdit::paintEvent(QPaintEvent* event)
     QPainter painter(this);
 
     painter.setCompositionMode(QPainter::CompositionMode_Multiply);
-
-    painter.fillRect(x(), y(), m_progress * width() / 100, height(), palette().highlight());
+    painter.fillRect(0, 0, m_progress * width() / 100, height(), palette().highlight());
 }
 
 void ProgressLineEdit::keyPressEvent(QKeyEvent* event)
@@ -237,3 +285,39 @@ void ProgressLineEdit::keyPressEvent(QKeyEvent* event)
         emit returnPressed(event->modifiers());
     }
 }
+
+// search line edit
+
+SearchLineEdit::SearchLineEdit(QWidget* parent) : ProgressLineEdit(parent)
+{
+    m_timer = new QTimer(this);
+
+    m_timer->setInterval(2000);
+    m_timer->setSingleShot(true);
+
+    connect(this, SIGNAL(textEdited(QString)), m_timer, SLOT(start()));
+    connect(this, SIGNAL(returnPressed(Qt::KeyboardModifiers)), SLOT(on_returnPressed(Qt::KeyboardModifiers)));
+    connect(m_timer, SIGNAL(timeout()), SLOT(on_timeout()));
+}
+
+void SearchLineEdit::startSearch()
+{
+    QTimer::singleShot(0, this, SLOT(on_timeout()));
+}
+
+void SearchLineEdit::stopTimer()
+{
+    m_timer->stop();
+}
+
+void SearchLineEdit::on_timeout()
+{
+    emit searchInitiated(text());
+}
+
+void SearchLineEdit::on_returnPressed(const Qt::KeyboardModifiers& modifiers)
+{
+    stopTimer();
+
+    emit searchInitiated(text(), modifiers == Qt::ShiftModifier);
+}
