@@ -58,6 +58,9 @@ public:
 protected:
     void mousePressEvent(QMouseEvent* event);
 
+private:
+    Q_DISABLE_COPY(TabBar)
+
 };
 
 // tab widget
@@ -79,11 +82,19 @@ public:
     TabBarPolicy tabBarPolicy() const;
     void setTabBarPolicy(TabBarPolicy tabBarPolicy);
 
+signals:
+    void tabContextMenuRequested(const QPoint& globalPos, int index);
+
+protected slots:
+    void on_tabBar_customContextMenuRequested(const QPoint& pos);
+
 protected:
     void tabInserted(int index);
     void tabRemoved(int index);
 
 private:
+    Q_DISABLE_COPY(TabWidget)
+
     TabBarPolicy m_tabBarPolicy;
 
 };
@@ -95,15 +106,28 @@ class TreeView : public QTreeView
     Q_OBJECT
 
 public:
-    explicit TreeView(QWidget* parent = 0);
+    explicit TreeView(int expansionRole, QWidget* parent = 0);
 
 public slots:
+    void expandAbove(const QModelIndex& child);
+
     void expandAll(const QModelIndex& index = QModelIndex());
     void collapseAll(const QModelIndex& index = QModelIndex());
 
+    void restoreExpansion(const QModelIndex& index = QModelIndex());
+
 protected:
     void contextMenuEvent(QContextMenuEvent* event);
 
+protected slots:
+    void on_expanded(const QModelIndex& index);
+    void on_collapsed(const QModelIndex& index);
+
+private:
+    Q_DISABLE_COPY(TreeView)
+
+    int m_expansionRole;
+
 };
 
 // line edit
@@ -118,6 +142,9 @@ public:
 protected:
     void mousePressEvent(QMouseEvent* event);
 
+private:
+    Q_DISABLE_COPY(LineEdit)
+
 };
 
 // combo box
@@ -129,6 +156,9 @@ class ComboBox : public QComboBox
 public:
     explicit ComboBox(QWidget* parent = 0);
 
+private:
+    Q_DISABLE_COPY(ComboBox)
+
 };
 
 // spin box
@@ -146,6 +176,9 @@ signals:
 protected:
     void keyPressEvent(QKeyEvent* event);
 
+private:
+    Q_DISABLE_COPY(SpinBox)
+
 };
 
 // progress line edit
@@ -168,8 +201,36 @@ protected:
     void keyPressEvent(QKeyEvent* event);
 
 private:
+    Q_DISABLE_COPY(ProgressLineEdit)
+
     int m_progress;
 
 };
 
+// search line edit
+
+class SearchLineEdit : public ProgressLineEdit
+{
+    Q_OBJECT
+
+public:
+    explicit SearchLineEdit(QWidget* parent = 0);
+
+    void startSearch();
+    void stopTimer();
+
+signals:
+    void searchInitiated(const QString& text, bool allTabs = false);
+
+protected slots:
+    void on_timeout();
+    void on_returnPressed(const Qt::KeyboardModifiers& modifiers);
+
+private:
+    Q_DISABLE_COPY(SearchLineEdit)
+
+    QTimer* m_timer;
+
+};
+
 #endif // MISCELLANEOUS_H
