@@ -24,6 +24,8 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QApplication>
 #include <QClipboard>
 #include <QFileDialog>
+#include <QGraphicsProxyWidget>
+#include <QGraphicsScene>
 #include <QGraphicsSceneHoverEvent>
 #include <qmath.h>
 #include <QMenu>
@@ -31,6 +33,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QPainter>
 #include <QTimer>
 #include <QToolTip>
+#include <QUrl>
 
 #include "settings.h"
 #include "model.h"
@@ -46,20 +49,26 @@ PageItem::PageItem(Model::Page* page, int index, bool presentationMode, QGraphic
     m_size(),
     m_links(),
     m_annotations(),
+    m_formFields(),
+    m_annotationOverlay(),
+    m_formFieldOverlay(),
     m_presentationMode(presentationMode),
     m_invertColors(false),
     m_highlights(),
     m_rubberBandMode(ModifiersMode),
     m_rubberBand(),
-    m_physicalDpiX(72),
-    m_physicalDpiY(72),
+    m_resolutionX(72),
+    m_resolutionY(72),
+    m_devicePixelRatio(1.0),
     m_scaleFactor(1.0),
     m_rotation(RotateBy0),
     m_transform(),
     m_normalizedTransform(),
     m_boundingRect(),
     m_pixmap(),
-    m_renderTask(0)
+    m_renderTask(0),
+    m_obsoletePixmap(),
+    m_obsoleteTransform()
 {
     if(s_settings == 0)
     {
@@ -73,7 +82,7 @@ PageItem::PageItem(Model::Page* page, int index, bool presentationMode, QGraphic
     m_renderTask = new RenderTask(this);
 
     connect(m_renderTask, SIGNAL(finished()), SLOT(on_renderTask_finished()));
-    connect(m_renderTask, SIGNAL(imageReady(int,int,qreal,Rotation,bool,bool,QImage)), SLOT(on_renderTask_imageReady(int,int,qreal,Rotation,bool,bool,QImage)));
+    connect(m_renderTask, SIGNAL(imageReady(int,int,qreal,qreal,Rotation,bool,bool,QImage)), SLOT(on_renderTask_imageReady(int,int,qreal,qreal,Rotation,bool,bool,QImage)));
 
     m_page = page;
 
@@ -87,6 +96,9 @@ PageItem::PageItem(Model::Page* page, int index, bool presentationMode, QGraphic
 
 PageItem::~PageItem()
 {
+    hideAnnotationOverlay(false);
+    hideFormFieldOverlay(false);
+
     m_renderTask->cancel();
     m_renderTask->wait();
 
@@ -104,131 +116,13 @@ QRectF PageItem::boundingRect() const
 
 void PageItem::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*)
 {
-    QPixmap pixmap;
+    paintPage(painter, cachedPixmap());
 
-    if(s_cache.contains(this))
-    {
-        pixmap = *s_cache.object(this);
-    }
-    else
-    {
-        if(!m_pixmap.isNull())
-        {
-            pixmap = m_pixmap;
-            m_pixmap = QPixmap();
+    paintLinks(painter);
+    paintFormFields(painter);
 
-            int cost = pixmap.width() * pixmap.height() * pixmap.depth() / 8;
-            s_cache.insert(this, new QPixmap(pixmap), cost);
-        }
-        else
-        {
-            startRender();
-        }
-    }
-
-    // page
-
-    if(s_settings->pageItem().decoratePages() && !m_presentationMode)
-    {
-        QColor paperColor = s_settings->pageItem().paperColor();
-
-        if(m_invertColors)
-        {
-            paperColor.setRgb(~paperColor.rgb());
-        }
-
-        painter->fillRect(m_boundingRect, QBrush(paperColor));
-    }
-
-    if(pixmap.isNull())
-    {
-        // progess icon
-
-        const qreal extent = qMin(0.1 * m_boundingRect.width(), 0.1 * m_boundingRect.height());
-        const QRectF rect(m_boundingRect.left() + 0.01 * m_boundingRect.width(), m_boundingRect.top() + 0.01 * m_boundingRect.height(), extent, extent);
-
-        s_settings->pageItem().progressIcon().paint(painter, rect.toRect());
-    }
-    else
-    {
-        painter->drawPixmap(m_boundingRect.topLeft(), pixmap);
-    }
-
-    if(s_settings->pageItem().decoratePages() && !m_presentationMode)
-    {
-        painter->drawRect(m_boundingRect);
-    }
-
-    // links
-
-    if(s_settings->pageItem().decorateLinks() && !m_presentationMode && !m_links.isEmpty())
-    {
-        painter->save();
-
-        painter->setTransform(m_normalizedTransform, true);
-        painter->setPen(QPen(Qt::red));
-
-        foreach(const Model::Link* link, m_links)
-        {
-            painter->drawPath(link->boundary);
-        }
-
-        painter->restore();
-    }
-
-    // form fields
-
-    if(s_settings->pageItem().decorateFormFields() && !m_presentationMode && !m_formFields.isEmpty())
-    {
-        painter->save();
-
-        painter->setTransform(m_normalizedTransform, true);
-        painter->setPen(QPen(Qt::blue));
-
-        foreach(const Model::FormField* formField, m_formFields)
-        {
-            painter->drawRect(formField->boundary());
-        }
-
-        painter->restore();
-    }
-
-    // highlights
-
-    if(!m_highlights.isEmpty())
-    {
-        painter->save();
-
-        painter->setTransform(m_transform, true);
-        painter->setPen(QPen(s_settings->pageItem().highlightColor()));
-        painter->setBrush(QBrush(s_settings->pageItem().highlightColor()));
-        painter->setCompositionMode(QPainter::CompositionMode_Multiply);
-
-        foreach(const QRectF highlight, m_highlights)
-        {
-            painter->drawRect(highlight.normalized());
-        }
-
-        painter->restore();
-    }
-
-    // rubber band
-
-    if(!m_rubberBand.isNull())
-    {
-        painter->save();
-
-        QPen pen;
-        pen.setColor(Qt::white);
-        pen.setStyle(Qt::DashLine);
-
-        painter->setPen(pen);
-        painter->setCompositionMode(QPainter::CompositionMode_Difference);
-
-        painter->drawRect(m_rubberBand);
-
-        painter->restore();
-    }
+    paintHighlights(painter);
+    paintRubberBand(painter);
 }
 
 int PageItem::index() const
@@ -287,24 +181,52 @@ void PageItem::setRubberBandMode(RubberBandMode rubberBandMode)
     }
 }
 
-int PageItem::physicalDpiX() const
+bool PageItem::showsAnnotationOverlay() const
 {
-    return m_physicalDpiX;
+    return !m_annotationOverlay.isEmpty();
 }
 
-int PageItem::physicalDpiY() const
+bool PageItem::showsFormFieldOverlay() const
 {
-    return m_physicalDpiY;
+    return !m_formFieldOverlay.isEmpty();
 }
 
-void PageItem::setPhysicalDpi(int physicalDpiX, int physicalDpiY)
+int PageItem::resolutionX() const
 {
-    if((m_physicalDpiX != physicalDpiX || m_physicalDpiY != physicalDpiY) && physicalDpiX > 0 && physicalDpiY > 0)
+    return m_resolutionX;
+}
+
+int PageItem::resolutionY() const
+{
+    return m_resolutionY;
+}
+
+void PageItem::setResolution(int resolutionX, int resolutionY)
+{
+    if((m_resolutionX != resolutionX || m_resolutionY != resolutionY) && resolutionX > 0 && resolutionY > 0)
     {
         refresh();
 
-        m_physicalDpiX = physicalDpiX;
-        m_physicalDpiY = physicalDpiY;
+        m_resolutionX = resolutionX;
+        m_resolutionY = resolutionY;
+
+        prepareGeometryChange();
+        prepareGeometry();
+    }
+}
+
+qreal PageItem::devicePixelRatio() const
+{
+    return m_devicePixelRatio;
+}
+
+void PageItem::setDevicePixelRatio(qreal devicePixelRatio)
+{
+    if(!qFuzzyCompare(m_devicePixelRatio, devicePixelRatio) && devicePixelRatio > 0.0)
+    {
+        refresh();
+
+        m_devicePixelRatio = devicePixelRatio;
 
         prepareGeometryChange();
         prepareGeometry();
@@ -318,7 +240,7 @@ qreal PageItem::scaleFactor() const
 
 void PageItem::setScaleFactor(qreal scaleFactor)
 {
-    if(m_scaleFactor != scaleFactor && scaleFactor > 0.0)
+    if(!qFuzzyCompare(m_scaleFactor, scaleFactor) && scaleFactor > 0.0)
     {
         refresh();
 
@@ -359,8 +281,16 @@ const QTransform& PageItem::normalizedTransform() const
 
 void PageItem::refresh()
 {
-    cancelRender();
+    m_renderTask->cancel();
 
+    if(s_settings->pageItem().keepObsoletePixmaps() && s_cache.contains(this))
+    {
+        m_obsoletePixmap = *s_cache.object(this);
+        m_obsoleteTopLeft = m_boundingRect.topLeft();
+        m_obsoleteTransform = m_transform.inverted();
+    }
+
+    m_pixmap = QPixmap();
     s_cache.remove(this);
 
     update();
@@ -375,7 +305,7 @@ void PageItem::startRender(bool prefetch)
 
     if(!m_renderTask->isRunning())
     {
-        m_renderTask->start(m_page, m_physicalDpiX, m_physicalDpiY, m_scaleFactor, m_rotation, m_invertColors, prefetch);
+        m_renderTask->start(m_page, m_resolutionX, m_resolutionY, effectiveDevicePixelRatio(), m_scaleFactor, m_rotation, m_invertColors, prefetch);
     }
 }
 
@@ -384,6 +314,7 @@ void PageItem::cancelRender()
     m_renderTask->cancel();
 
     m_pixmap = QPixmap();
+    m_obsoletePixmap = QPixmap();
 }
 
 void PageItem::on_renderTask_finished()
@@ -391,9 +322,9 @@ void PageItem::on_renderTask_finished()
     update();
 }
 
-void PageItem::on_renderTask_imageReady(int physicalDpiX, int physicalDpiY, qreal scaleFactor, Rotation rotation, bool invertColors, bool prefetch, QImage image)
+void PageItem::on_renderTask_imageReady(int resolutionX, int resolutionY, qreal devicePixelRatio, qreal scaleFactor, Rotation rotation, bool invertColors, bool prefetch, QImage image)
 {
-    if(m_physicalDpiX != physicalDpiX || m_physicalDpiY != physicalDpiY || !qFuzzyCompare(m_scaleFactor, scaleFactor) || m_rotation != rotation || m_invertColors != invertColors)
+    if(m_resolutionX != resolutionX || m_resolutionY != resolutionY || !qFuzzyCompare(effectiveDevicePixelRatio(), devicePixelRatio) || !qFuzzyCompare(m_scaleFactor, scaleFactor) || m_rotation != rotation || m_invertColors != invertColors)
     {
         return;
     }
@@ -426,6 +357,58 @@ void PageItem::on_renderTask_imageReady(int physicalDpiX, int physicalDpiY, qrea
             m_pixmap = QPixmap::fromImage(image);
         }
     }
+
+    m_obsoletePixmap = QPixmap();
+}
+
+void PageItem::showAnnotationOverlay(Model::Annotation* selectedAnnotation)
+{
+    if(s_settings->pageItem().annotationOverlay())
+    {
+        showOverlay(m_annotationOverlay, SLOT(hideAnnotationOverlay()), m_annotations, selectedAnnotation);
+    }
+    else
+    {
+        hideAnnotationOverlay(false);
+
+        addProxy(m_annotationOverlay, SLOT(hideAnnotationOverlay()), selectedAnnotation);
+        m_annotationOverlay.value(selectedAnnotation)->widget()->setFocus();
+    }
+}
+
+void PageItem::hideAnnotationOverlay(bool deleteLater)
+{
+    hideOverlay(m_annotationOverlay, deleteLater);
+}
+
+void PageItem::updateAnnotationOverlay()
+{
+    updateOverlay(m_annotationOverlay);
+}
+
+void PageItem::showFormFieldOverlay(Model::FormField* selectedFormField)
+{
+    if(s_settings->pageItem().formFieldOverlay())
+    {
+        showOverlay(m_formFieldOverlay, SLOT(hideFormFieldOverlay()), m_formFields, selectedFormField);
+    }
+    else
+    {
+        hideFormFieldOverlay(false);
+
+        addProxy(m_formFieldOverlay, SLOT(hideFormFieldOverlay()), selectedFormField);
+        m_formFieldOverlay.value(selectedFormField)->widget()->setFocus();
+    }
+}
+
+void PageItem::updateFormFieldOverlay()
+{
+    updateOverlay(m_formFieldOverlay);
+}
+
+void PageItem::hideFormFieldOverlay(bool deleteLater)
+{
+    hideOverlay(m_formFieldOverlay, deleteLater);
 }
 
 void PageItem::hoverEnterEvent(QGraphicsSceneHoverEvent*)
@@ -590,13 +573,15 @@ void PageItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
             {
                 unsetCursor();
 
-                editAnnotation(annotation, event->screenPos());
+                showAnnotationOverlay(annotation);
 
                 event->accept();
                 return;
             }
         }
 
+        hideAnnotationOverlay();
+
         // form fields
 
         foreach(Model::FormField* formField, m_formFields)
@@ -605,12 +590,14 @@ void PageItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
             {
                 unsetCursor();
 
-                editFormField(formField, event->screenPos());
+                showFormFieldOverlay(formField);
 
                 event->accept();
                 return;
             }
         }
+
+        hideFormFieldOverlay();
     }
 
     event->ignore();
@@ -701,7 +688,18 @@ void PageItem::loadInteractiveElements()
     if(!m_presentationMode)
     {
         m_annotations = m_page->annotations();
+
+        foreach(const Model::Annotation* annotation, m_annotations)
+        {
+            connect(annotation, SIGNAL(wasModified()), SIGNAL(wasModified()));
+        }
+
         m_formFields = m_page->formFields();
+
+        foreach(const Model::FormField* formField, m_formFields)
+        {
+            connect(formField, SIGNAL(wasModified()), SIGNAL(wasModified()));
+        }
     }
 
     update();
@@ -729,7 +727,7 @@ void PageItem::copyToClipboard(const QPoint& screenPos)
     else if(action == copyImageAction || action == saveImageToFileAction)
     {
         const QRect rect = m_rubberBand.translated(-m_boundingRect.topLeft()).toRect();
-        const QImage image = s_cache.contains(this) ? s_cache.object(this)->copy(rect).toImage() : m_page->render(m_physicalDpiX * m_scaleFactor, m_scaleFactor * m_physicalDpiY, m_rotation, rect);
+        const QImage image = s_cache.contains(this) ? s_cache.object(this)->copy(rect).toImage() : m_page->render(m_resolutionX * m_scaleFactor, m_scaleFactor * m_resolutionY, m_rotation, rect);
 
         if(!image.isNull())
         {
@@ -780,10 +778,12 @@ void PageItem::addAnnotation(const QPoint& screenPos)
             }
 
             m_annotations.append(annotation);
+            connect(annotation, SIGNAL(wasModified()), SIGNAL(wasModified()));
 
             refresh();
+            emit wasModified();
 
-            editAnnotation(annotation, screenPos);
+            showAnnotationOverlay(annotation);
         }
     }
 }
@@ -803,30 +803,166 @@ void PageItem::removeAnnotation(Model::Annotation* annotation, const QPoint& scr
             m_annotations.removeAll(annotation);
             m_page->removeAnnotation(annotation);
 
+            annotation->deleteLater();
+
             refresh();
+            emit wasModified();
         }
     }
 }
 
-void PageItem::editAnnotation(Model::Annotation* annotation, const QPoint& screenPos)
+template< typename Overlay, typename Element >
+void PageItem::showOverlay(Overlay& overlay, const char* hideOverlay, const QList< Element* >& elements, Element* selectedElement)
 {
-    annotation->showDialog(screenPos);
+    foreach(Element* element, elements)
+    {
+        if(!overlay.contains(element))
+        {
+            addProxy(overlay, hideOverlay, element);
+        }
+
+        if(element == selectedElement)
+        {
+            overlay.value(element)->widget()->setFocus();
+        }
+    }
 }
 
-void PageItem::editFormField(Model::FormField* formField, const QPoint& screenPos)
+template< typename Overlay, typename Element >
+void PageItem::addProxy(Overlay& overlay, const char* hideOverlay, Element* element)
 {
-    QDialog* formFieldDialog = formField->showDialog(screenPos);
+    QGraphicsProxyWidget* proxy = new QGraphicsProxyWidget(this);
+    proxy->setWidget(element->createWidget());
 
-    if(formFieldDialog != 0)
-    {
-        connect(formFieldDialog, SIGNAL(destroyed()), SLOT(refresh()));
-    }
-    else
+#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
+
+    proxy->setAutoFillBackground(true);
+
+#endif // QT_VERSION
+
+    overlay.insert(element, proxy);
+    setProxyGeometry(element, proxy);
+
+    connect(proxy, SIGNAL(visibleChanged()), hideOverlay);
+}
+
+template< typename Overlay >
+void PageItem::hideOverlay(Overlay& overlay, bool deleteLater)
+{
+    Overlay discardedOverlay;
+
+#if QT_VERSION >= QT_VERSION_CHECK(4,8,0)
+
+    discardedOverlay.swap(overlay);
+
+#else
+
+    discardedOverlay = overlay;
+    overlay = Overlay();
+
+#endif // QT_VERSION
+
+    if(!discardedOverlay.isEmpty())
     {
+        for(typename Overlay::const_iterator i = discardedOverlay.constBegin(); i != discardedOverlay.constEnd(); ++i)
+        {
+            if(deleteLater)
+            {
+                i.value()->deleteLater();
+            }
+            else
+            {
+                delete i.value();
+            }
+        }
+
         refresh();
     }
 }
 
+template< typename Overlay >
+void PageItem::updateOverlay(const Overlay& overlay) const
+{
+    for(typename Overlay::const_iterator i = overlay.constBegin(); i != overlay.constEnd(); ++i)
+    {
+        setProxyGeometry(i.key(), i.value());
+    }
+}
+
+void PageItem::setProxyGeometry(Model::Annotation* annotation, QGraphicsProxyWidget* proxy) const
+{
+    const QPointF center = m_normalizedTransform.map(annotation->boundary().center());
+
+    qreal x = center.x() - 0.5 * proxy->preferredWidth();
+    qreal y = center.y() - 0.5 * proxy->preferredHeight();
+    qreal width = proxy->preferredWidth();
+    qreal height = proxy->preferredHeight();
+
+    x = qMax(x, m_boundingRect.left() + proxyPadding);
+    y = qMax(y, m_boundingRect.top() + proxyPadding);
+
+    width = qMin(width, m_boundingRect.right() - proxyPadding - x);
+    height = qMin(height, m_boundingRect.bottom() - proxyPadding - y);
+
+    proxy->setGeometry(QRectF(x, y, width, height));
+}
+
+void PageItem::setProxyGeometry(Model::FormField* formField, QGraphicsProxyWidget* proxy) const
+{
+    QRectF rect = m_normalizedTransform.mapRect(formField->boundary());
+
+    qreal x = rect.x();
+    qreal y = rect.y();
+    qreal width = rect.width();
+    qreal height = rect.height();
+
+    switch(m_rotation)
+    {
+    default:
+    case RotateBy0:
+        proxy->setRotation(0.0);
+        break;
+    case RotateBy90:
+        x += width;
+        qSwap(width, height);
+
+        proxy->setRotation(90.0);
+        break;
+    case RotateBy180:
+        x += width;
+        y += height;
+
+        proxy->setRotation(180.0);
+        break;
+    case RotateBy270:
+        y += height;
+        qSwap(width, height);
+
+        proxy->setRotation(270.0);
+        break;
+    }
+
+    width /= m_scaleFactor;
+    height /= m_scaleFactor;
+
+    proxy->setScale(m_scaleFactor);
+
+    proxy->setGeometry(QRectF(x - proxyPadding, y - proxyPadding, width + proxyPadding, height + proxyPadding));
+}
+
+qreal PageItem::effectiveDevicePixelRatio()
+{
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+
+    return s_settings->pageItem().useDevicePixelRatio() ? m_devicePixelRatio : 1.0;
+
+#else
+
+    return 1.0;
+
+#endif // QT_VERSION
+}
+
 void PageItem::prepareGeometry()
 {
     m_transform.reset();
@@ -856,13 +992,13 @@ void PageItem::prepareGeometry()
     default:
     case RotateBy0:
     case RotateBy180:
-        m_transform.scale(m_scaleFactor * m_physicalDpiX / 72.0, m_scaleFactor * m_physicalDpiY / 72.0);
-        m_normalizedTransform.scale(m_scaleFactor * m_physicalDpiX / 72.0 * m_size.width(), m_scaleFactor * m_physicalDpiY / 72.0 * m_size.height());
+        m_transform.scale(m_scaleFactor * m_resolutionX / 72.0, m_scaleFactor * m_resolutionY / 72.0);
+        m_normalizedTransform.scale(m_scaleFactor * m_resolutionX / 72.0 * m_size.width(), m_scaleFactor * m_resolutionY / 72.0 * m_size.height());
         break;
     case RotateBy90:
     case RotateBy270:
-        m_transform.scale(m_scaleFactor * m_physicalDpiY / 72.0, m_scaleFactor * m_physicalDpiX / 72.0);
-        m_normalizedTransform.scale(m_scaleFactor * m_physicalDpiY / 72.0 * m_size.width(), m_scaleFactor * m_physicalDpiX / 72.0 * m_size.height());
+        m_transform.scale(m_scaleFactor * m_resolutionY / 72.0, m_scaleFactor * m_resolutionX / 72.0);
+        m_normalizedTransform.scale(m_scaleFactor * m_resolutionY / 72.0 * m_size.width(), m_scaleFactor * m_resolutionX / 72.0 * m_size.height());
         break;
     }
 
@@ -870,23 +1006,174 @@ void PageItem::prepareGeometry()
 
     m_boundingRect.setWidth(qRound(m_boundingRect.width()));
     m_boundingRect.setHeight(qRound(m_boundingRect.height()));
+
+    updateAnnotationOverlay();
+    updateFormFieldOverlay();
 }
 
-ThumbnailItem::ThumbnailItem(Model::Page* page, int index, QGraphicsItem* parent) : PageItem(page, index, false, parent)
+QPixmap PageItem::cachedPixmap()
+{
+    QPixmap pixmap;
+
+    if(s_cache.contains(this))
+    {
+        pixmap = *s_cache.object(this);
+    }
+    else
+    {
+        if(!m_pixmap.isNull())
+        {
+            pixmap = m_pixmap;
+            m_pixmap = QPixmap();
+
+            int cost = pixmap.width() * pixmap.height() * pixmap.depth() / 8;
+            s_cache.insert(this, new QPixmap(pixmap), cost);
+        }
+        else
+        {
+            startRender();
+        }
+    }
+
+    return pixmap;
+}
+
+void PageItem::paintPage(QPainter* painter, const QPixmap& pixmap) const
+{
+    if(s_settings->pageItem().decoratePages() && !m_presentationMode)
+    {
+        QColor paperColor = s_settings->pageItem().paperColor();
+
+        if(m_invertColors)
+        {
+            paperColor.setRgb(~paperColor.rgb());
+        }
+
+        painter->fillRect(m_boundingRect, QBrush(paperColor));
+    }
+
+    if(!pixmap.isNull())
+    {
+        painter->drawPixmap(m_boundingRect.topLeft(), pixmap);
+    }
+    else if(!m_obsoletePixmap.isNull())
+    {
+        painter->save();
+
+        painter->setTransform(m_obsoleteTransform, true);
+        painter->setTransform(m_transform, true);
+
+        painter->drawPixmap(m_obsoleteTopLeft, m_obsoletePixmap);
+
+        painter->restore();
+    }
+    else
+    {
+        // progess icon
+
+        const qreal extent = qMin(0.1 * m_boundingRect.width(), 0.1 * m_boundingRect.height());
+        const QRectF rect(m_boundingRect.left() + 0.01 * m_boundingRect.width(), m_boundingRect.top() + 0.01 * m_boundingRect.height(), extent, extent);
+
+        s_settings->pageItem().progressIcon().paint(painter, rect.toRect());
+    }
+
+    if(s_settings->pageItem().decoratePages() && !m_presentationMode)
+    {
+        painter->drawRect(m_boundingRect);
+    }
+}
+
+void PageItem::paintLinks(QPainter* painter) const
+{
+    if(s_settings->pageItem().decorateLinks() && !m_presentationMode && !m_links.isEmpty())
+    {
+        painter->save();
+
+        painter->setTransform(m_normalizedTransform, true);
+        painter->setPen(QPen(Qt::red, 0.0));
+
+        foreach(const Model::Link* link, m_links)
+        {
+            painter->drawPath(link->boundary);
+        }
+
+        painter->restore();
+    }
+}
+
+void PageItem::paintFormFields(QPainter* painter) const
+{
+    if(s_settings->pageItem().decorateFormFields() && !m_presentationMode && !m_formFields.isEmpty())
+    {
+        painter->save();
+
+        painter->setTransform(m_normalizedTransform, true);
+        painter->setPen(QPen(Qt::blue, 0.0));
+
+        foreach(const Model::FormField* formField, m_formFields)
+        {
+            painter->drawRect(formField->boundary());
+        }
+
+        painter->restore();
+    }
+}
+
+void PageItem::paintHighlights(QPainter* painter) const
+{
+    if(!m_highlights.isEmpty())
+    {
+        painter->save();
+
+        painter->setTransform(m_transform, true);
+        painter->setPen(QPen(s_settings->pageItem().highlightColor(), 0.0));
+        painter->setBrush(QBrush(s_settings->pageItem().highlightColor()));
+        painter->setCompositionMode(QPainter::CompositionMode_Multiply);
+
+        foreach(const QRectF highlight, m_highlights)
+        {
+            painter->drawRect(highlight.normalized());
+        }
+
+        painter->restore();
+    }
+}
+
+void PageItem::paintRubberBand(QPainter* painter) const
+{
+    if(!m_rubberBand.isNull())
+    {
+        painter->save();
+
+        painter->setPen(QPen(Qt::white, 0.0, Qt::DashLine));
+        painter->setCompositionMode(QPainter::CompositionMode_Difference);
+
+        painter->drawRect(m_rubberBand);
+
+        painter->restore();
+    }
+}
+
+ThumbnailItem::ThumbnailItem(Model::Page* page, int index, QGraphicsItem* parent) : PageItem(page, index, false, parent),
+#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
+    m_text(QString::number(index + 1)),
+#endif // QT_VERSION
+    m_current(false)
 {
     setAcceptHoverEvents(false);
-
-    const QFontMetrics fontMetrics = QFontMetrics(QFont());
-
-    m_textWidth = fontMetrics.width(QString::number(index + 1));
-    m_textHeight = fontMetrics.height();
-
-    m_current = false;
 }
 
 QRectF ThumbnailItem::boundingRect() const
 {
-    return PageItem::boundingRect().adjusted(0.0, 0.0, 0.0, 2.0 * m_textHeight);
+#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
+
+    return PageItem::boundingRect().adjusted(0.0, 0.0, 0.0, 2.0 * m_text.size().height());
+
+#else
+
+    return PageItem::boundingRect().adjusted(0.0, 0.0, 0.0, 2.0 * QFontMetrics(QFont()).height());
+
+#endif // QT_VERSION
 }
 
 void ThumbnailItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
@@ -895,18 +1182,34 @@ void ThumbnailItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* opt
 
     const QRectF boundingRect = PageItem::boundingRect();
 
-    QPointF pos = boundingRect.bottomLeft();
-    pos.rx() += 0.5 * (boundingRect.width() - m_textWidth);
-    pos.ry() += m_textHeight;
+#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
 
-    painter->drawText(pos, QString::number(index() + 1));
+    const QSizeF textSize = m_text.size();
+
+    QPointF pos = boundingRect.bottomLeft();
+    pos.rx() += 0.5 * (boundingRect.width() - textSize.width());
+    pos.ry() += 0.5 * textSize.height();
+
+    painter->drawStaticText(pos, m_text);
+
+#else
+
+    const QString text = QString::number(index() + 1);
+    const QFontMetrics fontMetrics = QFontMetrics(QFont());
+
+    QPointF pos = boundingRect.bottomLeft();
+    pos.rx() += 0.5 * (boundingRect.width() - fontMetrics.width(text));
+    pos.ry() += fontMetrics.height();
+
+    painter->drawText(pos, text);
+
+#endif // QT_VERSION
 
     if(m_current)
     {
         painter->save();
 
         painter->setCompositionMode(QPainter::CompositionMode_Multiply);
-
         painter->fillRect(boundingRect, widget->palette().highlight());
 
         painter->restore();
@@ -951,4 +1254,46 @@ void ThumbnailItem::contextMenuEvent(QGraphicsSceneContextMenuEvent*)
 
 void ThumbnailItem::loadInteractiveElements()
 {
+    const qreal width = size().width() / 72.0 * 25.4;
+    const qreal height = size().height() / 72.0 * 25.4;
+
+    const qreal longEdge = qMax(width, height);
+    const qreal shortEdge = qMin(width, height);
+
+    QString paperSize;
+
+    if(qAbs(longEdge - 279.4) <= 1.0 && qAbs(shortEdge - 215.9) <= 1.0)
+    {
+        paperSize = QLatin1String(" (Letter)");
+    }
+    else
+    {
+        qreal longEdgeA = 1189.0;
+        qreal shortEdgeA = 841.0;
+
+        qreal longEdgeB = 1414.0;
+        qreal shortEdgeB = 1000.0;
+
+        for(int i = 0; i <= 10; ++i)
+        {
+            if(qAbs(longEdge - longEdgeA) <= 1.0 && qAbs(shortEdge - shortEdgeA) <= 1.0)
+            {
+                paperSize = QString(" (A%1)").arg(i);
+                break;
+            }
+            else if(qAbs(longEdge - longEdgeB) <= 1.0 && qAbs(shortEdge - shortEdgeB) <= 1.0)
+            {
+                paperSize = QString(" (B%1)").arg(i);
+                break;
+            }
+
+            longEdgeA = shortEdgeA;
+            shortEdgeA /= qSqrt(2.0);
+
+            longEdgeB = shortEdgeB;
+            shortEdgeB /= qSqrt(2.0);
+        }
+    }
+
+    setToolTip(QString("%1 mm x %2 mm%3").arg(width, 0, 'f', 1).arg(height, 0, 'f', 1).arg(paperSize));
 }
