@@ -26,6 +26,14 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QGraphicsObject>
 #include <QIcon>
 
+#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
+
+#include <QStaticText>
+
+#endif // QT_VERSION
+
+class QGraphicsProxyWidget;
+
 #include "global.h"
 
 namespace Model
@@ -62,9 +70,15 @@ public:
     RubberBandMode rubberBandMode() const;
     void setRubberBandMode(RubberBandMode rubberBandMode);
 
-    int physicalDpiX() const;
-    int physicalDpiY() const;
-    void setPhysicalDpi(int physicalDpiX, int physicalDpiY);
+    bool showsAnnotationOverlay() const;
+    bool showsFormFieldOverlay() const;
+
+    int resolutionX() const;
+    int resolutionY() const;
+    void setResolution(int resolutionX, int resolutionY);
+
+    qreal devicePixelRatio() const;
+    void setDevicePixelRatio(qreal devicePixelRatio);
 
     qreal scaleFactor() const;
     void setScaleFactor(qreal scaleFactor);
@@ -85,6 +99,8 @@ signals:
 
     void sourceRequested(int page, const QPointF& pos);
 
+    void wasModified();
+
 public slots:
     void refresh();
 
@@ -93,7 +109,15 @@ public slots:
 
 protected slots:
     void on_renderTask_finished();
-    void on_renderTask_imageReady(int physicalDpiX, int physicalDpiY, qreal scaleFactor, Rotation rotation, bool invertColors, bool prefetch, QImage image);
+    void on_renderTask_imageReady(int resolutionX, int resolutionY, qreal devicePixelRatio, qreal scaleFactor, Rotation rotation, bool invertColors, bool prefetch, QImage image);
+
+    void showAnnotationOverlay(Model::Annotation* selectedAnnotation);
+    void hideAnnotationOverlay(bool deleteLater = true);
+    void updateAnnotationOverlay();
+
+    void showFormFieldOverlay(Model::FormField* selectedFormField);
+    void hideFormFieldOverlay(bool deleteLater = true);
+    void updateFormFieldOverlay();
 
 protected:
     void hoverEnterEvent(QGraphicsSceneHoverEvent*);
@@ -111,6 +135,8 @@ private slots:
     virtual void loadInteractiveElements();
 
 private:
+    Q_DISABLE_COPY(PageItem)
+
     static Settings* s_settings;
 
     static QCache< PageItem*, QPixmap > s_cache;
@@ -124,6 +150,12 @@ private:
     QList< Model::Annotation* > m_annotations;
     QList< Model::FormField* > m_formFields;
 
+    typedef QMap< Model::Annotation*, QGraphicsProxyWidget* > AnnotationOverlay;
+    AnnotationOverlay m_annotationOverlay;
+
+    typedef QMap< Model::FormField*, QGraphicsProxyWidget* > FormFieldOverlay;
+    FormFieldOverlay m_formFieldOverlay;
+
     bool m_presentationMode;
     bool m_invertColors;
 
@@ -136,14 +168,23 @@ private:
 
     void addAnnotation(const QPoint& screenPos);
     void removeAnnotation(Model::Annotation* annotation, const QPoint& screenPos);
-    void editAnnotation(Model::Annotation* annotation, const QPoint& screenPos);
 
-    void editFormField(Model::FormField* formField, const QPoint& screenPos);
+    template< typename Overlay, typename Element > void showOverlay(Overlay& overlay, const char* hideOverlay, const QList< Element* >& elements, Element* selectedElement);
+    template< typename Overlay, typename Element > void addProxy(Overlay& overlay, const char* hideOverlay, Element* element);
+    template< typename Overlay > void hideOverlay(Overlay& overlay, bool deleteLater);
+    template< typename Overlay > void updateOverlay(const Overlay& overlay) const;
+
+    static const qreal proxyPadding = 2.0;
+    void setProxyGeometry(Model::Annotation* annotation, QGraphicsProxyWidget* proxy) const;
+    void setProxyGeometry(Model::FormField* formField, QGraphicsProxyWidget* proxy) const;
 
     // geometry
 
-    int m_physicalDpiX;
-    int m_physicalDpiY;
+    int m_resolutionX;
+    int m_resolutionY;
+    qreal m_devicePixelRatio;
+
+    qreal effectiveDevicePixelRatio();
 
     qreal m_scaleFactor;
     Rotation m_rotation;
@@ -158,6 +199,23 @@ private:
 
     RenderTask* m_renderTask;
 
+    // obsolete pixmap
+
+    QPixmap m_obsoletePixmap;
+    QPointF m_obsoleteTopLeft;
+    QTransform m_obsoleteTransform;
+
+    // paint
+
+    QPixmap cachedPixmap();
+    void paintPage(QPainter* painter, const QPixmap& pixmap) const;
+
+    void paintLinks(QPainter* painter) const;
+    void paintFormFields(QPainter* painter) const;
+
+    void paintHighlights(QPainter* painter) const;
+    void paintRubberBand(QPainter* painter) const;
+
 };
 
 class ThumbnailItem : public PageItem
@@ -185,8 +243,13 @@ private slots:
     void loadInteractiveElements();
 
 private:
-    int m_textWidth;
-    int m_textHeight;
+    Q_DISABLE_COPY(ThumbnailItem)
+
+#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
+
+    QStaticText m_text;
+
+#endif // QT_VERSION
 
     bool m_current;
 
