@@ -21,25 +21,36 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "pdfmodel.h"
 
-#include <QCheckBox>
-#include <QComboBox>
 #include <QFormLayout>
 #include <QMessageBox>
 #include <QSettings>
-#include <QStandardItemModel>
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+
+#include <poppler-qt5.h>
+
+#else
 
 #include <poppler-qt4.h>
+
+#endif // QT_VERSION
+
 #include <poppler-form.h>
 
-#include "annotationdialog.h"
-#include "formfielddialog.h"
+#include "annotationwidgets.h"
+#include "formfieldwidgets.h"
 
-Model::PdfAnnotation::PdfAnnotation(QMutex* mutex, Poppler::Annotation* annotation) :
+Model::PdfAnnotation::PdfAnnotation(QMutex* mutex, Poppler::Annotation* annotation) : Annotation(),
     m_mutex(mutex),
     m_annotation(annotation)
 {
 }
 
+Model::PdfAnnotation::~PdfAnnotation()
+{
+    delete m_annotation;
+}
+
 QRectF Model::PdfAnnotation::boundary() const
 {
 #ifndef HAS_POPPLER_24
@@ -62,30 +73,37 @@ QString Model::PdfAnnotation::contents() const
     return m_annotation->contents();
 }
 
-QDialog* Model::PdfAnnotation::showDialog(const QPoint& screenPos)
+QWidget* Model::PdfAnnotation::createWidget()
 {
-#ifndef HAS_POPPLER_24
+    QWidget* widget = 0;
 
-    QMutexLocker mutexLocker(m_mutex);
+    if(m_annotation->subType() == Poppler::Annotation::AText || m_annotation->subType() == Poppler::Annotation::AHighlight)
+    {
+        widget = new AnnotationWidget(m_mutex, m_annotation);
 
-#endif // HAS_POPPLER_24
+        connect(widget, SIGNAL(wasModified()), SIGNAL(wasModified()));
+    }
+    else if(m_annotation->subType() == Poppler::Annotation::AFileAttachment)
+    {
+        widget = new FileAttachmentAnnotationWidget(m_mutex, static_cast< Poppler::FileAttachmentAnnotation* >(m_annotation));
+    }
 
-    AnnotationDialog* annotationDialog = new AnnotationDialog(m_mutex, m_annotation.data());
+    connect(this, SIGNAL(destroyed()), widget, SLOT(deleteLater()));
 
-    annotationDialog->move(screenPos);
-
-    annotationDialog->setAttribute(Qt::WA_DeleteOnClose);
-    annotationDialog->show();
-
-    return annotationDialog;
+    return widget;
 }
 
-Model::PdfFormField::PdfFormField(QMutex* mutex, Poppler::FormField* formField) :
+Model::PdfFormField::PdfFormField(QMutex* mutex, Poppler::FormField* formField) : FormField(),
     m_mutex(mutex),
     m_formField(formField)
 {
 }
 
+Model::PdfFormField::~PdfFormField()
+{
+    delete m_formField;
+}
+
 QRectF Model::PdfFormField::boundary() const
 {
 #ifndef HAS_POPPLER_24
@@ -108,33 +126,53 @@ QString Model::PdfFormField::name() const
     return m_formField->name();
 }
 
-QDialog* Model::PdfFormField::showDialog(const QPoint& screenPos)
+QWidget* Model::PdfFormField::createWidget()
 {
-#ifndef HAS_POPPLER_24
+    QWidget* widget = 0;
 
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    if(m_formField->type() == Poppler::FormField::FormText || m_formField->type() == Poppler::FormField::FormChoice)
+    if(m_formField->type() == Poppler::FormField::FormText)
     {
-        FormFieldDialog* formFieldDialog = new FormFieldDialog(m_mutex, m_formField.data());
+        Poppler::FormFieldText* formFieldText = static_cast< Poppler::FormFieldText* >(m_formField);
 
-        formFieldDialog->move(screenPos);
+        if(formFieldText->textType() == Poppler::FormFieldText::Normal)
+        {
+            widget = new NormalTextFieldWidget(m_mutex, formFieldText);
+        }
+        else if(formFieldText->textType() == Poppler::FormFieldText::Multiline)
+        {
+            widget = new MultilineTextFieldWidget(m_mutex, formFieldText);
+        }
+    }
+    else if(m_formField->type() == Poppler::FormField::FormChoice)
+    {
+        Poppler::FormFieldChoice* formFieldChoice = static_cast< Poppler::FormFieldChoice* >(m_formField);
 
-        formFieldDialog->setAttribute(Qt::WA_DeleteOnClose);
-        formFieldDialog->show();
-
-        return formFieldDialog;
+        if(formFieldChoice->choiceType() == Poppler::FormFieldChoice::ComboBox)
+        {
+            widget = new ComboBoxChoiceFieldWidget(m_mutex, formFieldChoice);
+        }
+        else if(formFieldChoice->choiceType() == Poppler::FormFieldChoice::ListBox)
+        {
+            widget = new ListBoxChoiceFieldWidget(m_mutex, formFieldChoice);
+        }
     }
     else if(m_formField->type() == Poppler::FormField::FormButton)
     {
-        Poppler::FormFieldButton* formFieldButton = static_cast< Poppler::FormFieldButton* >(m_formField.data());
+        Poppler::FormFieldButton* formFieldButton = static_cast< Poppler::FormFieldButton* >(m_formField);
 
-        formFieldButton->setState(!formFieldButton->state());
+        if(formFieldButton->buttonType() == Poppler::FormFieldButton::CheckBox)
+        {
+            widget = new CheckBoxChoiceFieldWidget(m_mutex, formFieldButton);
+        }
+        else if(formFieldButton->buttonType() == Poppler::FormFieldButton::Radio)
+        {
+            widget = new RadioChoiceFieldWidget(m_mutex, formFieldButton);
+        }
     }
 
-    return 0;
+    connect(widget, SIGNAL(wasModified()), SIGNAL(wasModified()));
+
+    return widget;
 }
 
 Model::PdfPage::PdfPage(QMutex* mutex, Poppler::Page* page) :
@@ -266,6 +304,15 @@ QList< Model::Link* > Model::PdfPage::links() const
 
             links.append(new Link(boundary, url));
         }
+        else if(link->linkType() == Poppler::Link::Execute)
+        {
+            const Poppler::LinkExecute* linkExecute = static_cast< const Poppler::LinkExecute* >(link);
+
+            const QRectF boundary = linkExecute->linkArea().normalized();
+            const QString url = linkExecute->fileName();
+
+            links.append(new Link(boundary, url));
+        }
 
         delete link;
     }
@@ -339,7 +386,7 @@ QList< Model::Annotation* > Model::PdfPage::annotations() const
 
     foreach(Poppler::Annotation* annotation, m_page->annotations())
     {
-        if(annotation->subType() == Poppler::Annotation::AText || annotation->subType() == Poppler::Annotation::AHighlight)
+        if(annotation->subType() == Poppler::Annotation::AText || annotation->subType() == Poppler::Annotation::AHighlight || annotation->subType() == Poppler::Annotation::AFileAttachment)
         {
             annotations.append(new PdfAnnotation(m_mutex, annotation));
             continue;
@@ -395,6 +442,7 @@ Model::Annotation* Model::PdfPage::addTextAnnotation(const QRectF& boundary, con
 #else
 
     Q_UNUSED(boundary);
+    Q_UNUSED(color);
 
     return 0;
 
@@ -438,6 +486,7 @@ Model::Annotation* Model::PdfPage::addHighlightAnnotation(const QRectF& boundary
 #else
 
     Q_UNUSED(boundary);
+    Q_UNUSED(color);
 
     return 0;
 
@@ -455,7 +504,10 @@ void Model::PdfPage::removeAnnotation(Annotation* annotation)
 
 #ifdef HAS_POPPLER_20
 
-    m_page->removeAnnotation(static_cast< PdfAnnotation* >(annotation)->m_annotation.data());
+    PdfAnnotation* pdfAnnotation = static_cast< PdfAnnotation* >(annotation);
+
+    m_page->removeAnnotation(pdfAnnotation->m_annotation);
+    pdfAnnotation->m_annotation = 0;
 
 #else
 
@@ -482,44 +534,38 @@ QList< Model::FormField* > Model::PdfPage::formFields() const
             continue;
         }
 
-        switch(formField->type())
+        if(formField->type() == Poppler::FormField::FormText)
         {
-        default:
-        case Poppler::FormField::FormSignature:
-            delete formField;
-            break;
-        case Poppler::FormField::FormText:
-            switch(static_cast< Poppler::FormFieldText* >(formField)->textType())
-            {
-            default:
-            case Poppler::FormFieldText::FileSelect:
-                delete formField;
-                break;
-            case Poppler::FormFieldText::Normal:
-            case Poppler::FormFieldText::Multiline:
-                formFields.append(new PdfFormField(m_mutex, formField));
-                break;
-            }
+            Poppler::FormFieldText* formFieldText = static_cast< Poppler::FormFieldText* >(formField);
 
-            break;
-        case Poppler::FormField::FormChoice:
-            formFields.append(new PdfFormField(m_mutex, formField));
-            break;
-        case Poppler::FormField::FormButton:
-            switch(static_cast< Poppler::FormFieldButton* >(formField)->buttonType())
+            if(formFieldText->textType() == Poppler::FormFieldText::Normal || formFieldText->textType() == Poppler::FormFieldText::Multiline)
             {
-            default:
-            case Poppler::FormFieldButton::Push:
-                delete formField;
-                break;
-            case Poppler::FormFieldButton::CheckBox:
-            case Poppler::FormFieldButton::Radio:
                 formFields.append(new PdfFormField(m_mutex, formField));
-                break;
+                continue;
             }
-
-            break;
         }
+        else if(formField->type() == Poppler::FormField::FormChoice)
+        {
+            Poppler::FormFieldChoice* formFieldChoice = static_cast< Poppler::FormFieldChoice* >(formField);
+
+            if(formFieldChoice->choiceType() == Poppler::FormFieldChoice::ListBox || formFieldChoice->choiceType() == Poppler::FormFieldChoice::ComboBox)
+            {
+                formFields.append(new PdfFormField(m_mutex, formField));
+                continue;
+            }
+        }
+        else if(formField->type() == Poppler::FormField::FormButton)
+        {
+            Poppler::FormFieldButton* formFieldButton = static_cast< Poppler::FormFieldButton* >(formField);
+
+            if(formFieldButton->buttonType() == Poppler::FormFieldButton::CheckBox || formFieldButton->buttonType() == Poppler::FormFieldButton::Radio)
+            {
+                formFields.append(new PdfFormField(m_mutex, formField));
+                continue;
+            }
+        }
+
+        delete formField;
     }
 
     return formFields;
@@ -633,7 +679,7 @@ static void loadOutline(Poppler::Document* document, const QDomNode& node, QStan
     const QDomElement element = node.toElement();
 
     QStandardItem* item = new QStandardItem(element.tagName());
-    item->setFlags(Qt::ItemIsEnabled);
+    item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
 
     Poppler::LinkDestination* linkDestination = 0;
 
@@ -677,8 +723,8 @@ static void loadOutline(Poppler::Document* document, const QDomNode& node, QStan
         item->setData(left, Qt::UserRole + 2);
         item->setData(top, Qt::UserRole + 3);
 
-        QStandardItem* pageItem = new QStandardItem(QString::number(page));
-        pageItem->setFlags(Qt::NoItemFlags);
+        QStandardItem* pageItem = item->clone();
+        pageItem->setText(QString::number(page));
         pageItem->setTextAlignment(Qt::AlignRight);
 
         parent->appendRow(QList< QStandardItem* >() << item << pageItem);
