@@ -391,7 +391,7 @@ void PresentationView::preparePages()
     {
         PageItem* page = new PageItem(m_pages.at(index), index, true);
 
-        page->setPhysicalDpi(physicalDpiX(), physicalDpiY());
+        page->setInvertColors(m_invertColors);
 
         scene()->addItem(page);
         m_pageItems.append(page);
@@ -402,7 +402,12 @@ void PresentationView::preparePages()
 
 void PresentationView::prepareBackground()
 {
-    QColor backgroundColor = s_settings->pageItem().paperColor();
+    QColor backgroundColor = s_settings->presentationView().backgroundColor();
+
+    if(!backgroundColor.isValid())
+    {
+        backgroundColor = s_settings->pageItem().paperColor();
+    }
 
     if(m_invertColors)
     {
@@ -418,6 +423,14 @@ void PresentationView::prepareScene()
     {
         PageItem* page = m_pageItems.at(index);
 
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+
+        page->setDevicePixelRatio(devicePixelRatio());
+
+#endif // QT_VERSION
+
+        page->setResolution(logicalDpiX(), logicalDpiY());
+
         const qreal visibleWidth = viewport()->width();
         const qreal visibleHeight = viewport()->height();
 
@@ -429,13 +442,13 @@ void PresentationView::prepareScene()
         default:
         case RotateBy0:
         case RotateBy180:
-            pageWidth = physicalDpiX() / 72.0 * page->size().width();
-            pageHeight = physicalDpiY() / 72.0 * page->size().height();
+            pageWidth = logicalDpiX() / 72.0 * page->size().width();
+            pageHeight = logicalDpiY() / 72.0 * page->size().height();
             break;
         case RotateBy90:
         case RotateBy270:
-            pageWidth = physicalDpiX() / 72.0 * page->size().height();
-            pageHeight = physicalDpiY() / 72.0 * page->size().width();
+            pageWidth = logicalDpiX() / 72.0 * page->size().height();
+            pageHeight = logicalDpiY() / 72.0 * page->size().width();
             break;
         }
 
