@@ -29,8 +29,9 @@ RenderTask::RenderTask(QObject* parent) : QObject(parent), QRunnable(),
     m_isRunning(false),
     m_wasCanceled(false),
     m_page(0),
-    m_physicalDpiX(72),
-    m_physicalDpiY(72),
+    m_resolutionX(72),
+    m_resolutionY(72),
+    m_devicePixelRatio(1.0),
     m_scaleFactor(1.0),
     m_rotation(RotateBy0),
     m_invertColors(false),
@@ -51,6 +52,8 @@ void RenderTask::wait()
 
 bool RenderTask::isRunning() const
 {
+    QMutexLocker mutexLocker(&m_mutex);
+
     return m_isRunning;
 }
 
@@ -68,7 +71,35 @@ void RenderTask::run()
         return;
     }
 
-    QImage image = m_page->render(m_physicalDpiX * m_scaleFactor, m_physicalDpiY * m_scaleFactor, m_rotation);
+    qreal resolutionX;
+    qreal resolutionY;
+
+    switch(m_rotation)
+    {
+    default:
+    case RotateBy0:
+    case RotateBy180:
+        resolutionX = m_resolutionX * m_scaleFactor;
+        resolutionY = m_resolutionY * m_scaleFactor;
+        break;
+    case RotateBy90:
+    case RotateBy270:
+        resolutionX = m_resolutionY * m_scaleFactor;
+        resolutionY = m_resolutionX * m_scaleFactor;
+        break;
+    }
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+
+    QImage image = m_page->render(m_devicePixelRatio * resolutionX, m_devicePixelRatio * resolutionY, m_rotation);
+
+    image.setDevicePixelRatio(m_devicePixelRatio);
+
+#else
+
+    QImage image = m_page->render(resolutionX, resolutionY, m_rotation);
+
+#endif // QT_VERSION
 
     if(m_wasCanceled && !m_prefetch)
     {
@@ -82,17 +113,18 @@ void RenderTask::run()
         image.invertPixels();
     }
 
-    emit imageReady(m_physicalDpiX, m_physicalDpiY, m_scaleFactor, m_rotation, m_invertColors, m_prefetch, image);
+    emit imageReady(m_resolutionX, m_resolutionY, m_devicePixelRatio, m_scaleFactor, m_rotation, m_invertColors, m_prefetch, image);
 
     finish();
 }
 
-void RenderTask::start(Model::Page* page, int physicalDpiX, int physicalDpiY, qreal scaleFactor, Rotation rotation, bool invertColors, bool prefetch)
+void RenderTask::start(Model::Page* page, int resolutionX, int resolutionY, qreal devicePixelRatio, qreal scaleFactor, Rotation rotation, bool invertColors, bool prefetch)
 {
     m_page = page;
 
-    m_physicalDpiX = physicalDpiX;
-    m_physicalDpiY = physicalDpiY;
+    m_resolutionX = resolutionX;
+    m_resolutionY = resolutionY;
+    m_devicePixelRatio = devicePixelRatio;
 
     m_scaleFactor = scaleFactor;
     m_rotation = rotation;
@@ -101,9 +133,10 @@ void RenderTask::start(Model::Page* page, int physicalDpiX, int physicalDpiY, qr
 
     m_prefetch = prefetch;
 
-    QMutexLocker mutexLocker(&m_mutex);
-
+    m_mutex.lock();
     m_isRunning = true;
+    m_mutex.unlock();
+
     m_wasCanceled = false;
 
     QThreadPool::globalInstance()->start(this);
@@ -116,11 +149,11 @@ void RenderTask::cancel()
 
 void RenderTask::finish()
 {
-    emit finished();
-
-    QMutexLocker mutexLocker(&m_mutex);
-
+    m_mutex.lock();
     m_isRunning = false;
+    m_mutex.unlock();
 
     m_waitCondition.wakeAll();
+
+    emit finished();
 }
