@@ -62,6 +62,9 @@ void Settings::PageItem::sync()
     m_progressIcon = QIcon::fromTheme("image-loading", QIcon(":/icons/image-loading.svg"));
     m_errorIcon = QIcon::fromTheme("image-missing", QIcon(":icons/image-missing.svg"));
 
+    m_keepObsoletePixmaps = m_settings->value("pageItem/keepObsoletePixmaps", Defaults::PageItem::keepObsoletePixmaps()).toBool();
+    m_useDevicePixelRatio = m_settings->value("pageItem/useDevicePixelRatio", Defaults::PageItem::useDevicePixelRatio()).toBool();
+
     m_decoratePages = m_settings->value("pageItem/decoratePages", Defaults::PageItem::decoratePages()).toBool();
     m_decorateLinks = m_settings->value("pageItem/decorateLinks", Defaults::PageItem::decorateLinks()).toBool();
     m_decorateFormFields = m_settings->value("pageItem/decorateFormFields", Defaults::PageItem::decorateFormFields()).toBool();
@@ -81,6 +84,18 @@ void Settings::PageItem::setCacheSize(int cacheSize)
     }
 }
 
+void Settings::PageItem::setKeepObsoletePixmaps(bool keepObsoletePixmaps)
+{
+    m_keepObsoletePixmaps = keepObsoletePixmaps;
+    m_settings->setValue("pageItem/keepObsoletePixmaps", keepObsoletePixmaps);
+}
+
+void Settings::PageItem::setUseDevicePixelRatio(bool useDevicePixelRatio)
+{
+    m_useDevicePixelRatio = useDevicePixelRatio;
+    m_settings->setValue("pageItem/useDevicePixelRatio", useDevicePixelRatio);
+}
+
 void Settings::PageItem::setDecoratePages(bool decorate)
 {
     m_decoratePages = decorate;
@@ -147,11 +162,32 @@ void Settings::PageItem::setAddAnnotationModifiers(const Qt::KeyboardModifiers&
     m_settings->setValue("pageItem/addAnnotationModifiers", static_cast< int >(modifiers));
 }
 
+bool Settings::PageItem::annotationOverlay() const
+{
+    return m_settings->value("pageItem/annotationOverlay", Defaults::PageItem::annotationOverlay()).toBool();
+}
+
+void Settings::PageItem::setAnnotationOverlay(bool overlay)
+{
+    m_settings->setValue("pageItem/annotationOverlay", overlay);
+}
+
+bool Settings::PageItem::formFieldOverlay() const
+{
+    return m_settings->value("pageItem/formFieldOverlay", Defaults::PageItem::formFieldOverlay()).toBool();
+}
+
+void Settings::PageItem::setFormFieldOverlay(bool overlay)
+{
+    m_settings->setValue("pageItem/formFieldOverlay", overlay);
+}
+
 Settings::PageItem::PageItem(QSettings* settings) :
     m_settings(settings),
     m_cacheSize(Defaults::PageItem::cacheSize()),
     m_progressIcon(),
     m_errorIcon(),
+    m_keepObsoletePixmaps(Defaults::PageItem::keepObsoletePixmaps()),
     m_decoratePages(Defaults::PageItem::decoratePages()),
     m_decorateLinks(Defaults::PageItem::decorateLinks()),
     m_decorateFormFields(Defaults::PageItem::decorateFormFields()),
@@ -190,6 +226,16 @@ void Settings::PresentationView::setScreen(int screen)
     m_settings->setValue("presentationView/screen", screen);
 }
 
+QColor Settings::PresentationView::backgroundColor() const
+{
+    return m_settings->value("presentationView/backgroundColor", Defaults::PresentationView::backgroundColor()).value< QColor >();
+}
+
+void Settings::PresentationView::setBackgroundColor(const QColor& backgroundColor)
+{
+    m_settings->setValue("presentationView/backgroundColor", backgroundColor);
+}
+
 Settings::PresentationView::PresentationView(QSettings* settings) :
     m_settings(settings)
 {
@@ -466,6 +512,16 @@ void Settings::MainWindow::setTrackRecentlyUsed(bool on)
     m_settings->setValue("mainWindow/trackRecentlyUsed", on);
 }
 
+int Settings::MainWindow::recentlyUsedCount() const
+{
+    return m_settings->value("mainWindow/recentlyUsedCount", Defaults::MainWindow::recentlyUsedCount()).toInt();
+}
+
+void Settings::MainWindow::setRecentlyUsedCount(int recentlyUsedCount)
+{
+    m_settings->setValue("mainWindow/recentlyUsedCount", recentlyUsedCount);
+}
+
 QStringList Settings::MainWindow::recentlyUsed() const
 {
     return m_settings->value("mainWindow/recentlyUsed").toStringList();
@@ -552,6 +608,26 @@ void Settings::MainWindow::setCurrentPageInWindowTitle(bool currentPageInTabText
     m_settings->setValue("mainWindow/currentPageInWindowTitle", currentPageInTabText);
 }
 
+bool Settings::MainWindow::instanceNameInWindowTitle() const
+{
+    return m_settings->value("mainWindow/instanceNameInWindowTitle", Defaults::MainWindow::instancfeNameInWindowTitle()).toBool();
+}
+
+void Settings::MainWindow::setInstanceNameInWindowTitle(bool instanceNameInWindowTitle)
+{
+    m_settings->setValue("mainWindow/instanceNameInWindowTitle", instanceNameInWindowTitle);
+}
+
+bool Settings::MainWindow::synchronizeOutlineView() const
+{
+    return m_settings->value("mainWindow/synchronizeOutlineView", Defaults::MainWindow::synchronizeOutlineView()).toBool();
+}
+
+void Settings::MainWindow::setSynchronizeOutlineView(bool synchronizeOutlineView)
+{
+    m_settings->setValue("mainWindow/synchronizeOutlineView", synchronizeOutlineView);
+}
+
 static QStringList trimmed(const QStringList& list)
 {
     QStringList trimmedList;
@@ -654,6 +730,16 @@ void Settings::MainWindow::setSavePath(const QString& savePath)
     m_settings->setValue("mainWindow/savePath", savePath);
 }
 
+QSize Settings::MainWindow::settingsDialogSize(const QSize& sizeHint) const
+{
+    return m_settings->value("mainWindow/settingsDialogSize", sizeHint).toSize();
+}
+
+void Settings::MainWindow::setSettingsDialogSize(const QSize& settingsDialogSize)
+{
+    m_settings->setValue("mainWindow/settingsDialogSize", settingsDialogSize);
+}
+
 QSize Settings::MainWindow::fontsDialogSize(const QSize& sizeHint) const
 {
     return m_settings->value("mainWindow/fontsDialogSize", sizeHint).toSize();
