@@ -55,6 +55,12 @@ public:
         inline const QIcon& progressIcon() const { return m_progressIcon; }
         inline const QIcon& errorIcon() const { return m_errorIcon; }
 
+        inline bool keepObsoletePixmaps() const { return m_keepObsoletePixmaps; }
+        void setKeepObsoletePixmaps(bool keepObsoletePixmaps);
+
+        inline bool useDevicePixelRatio() const { return m_useDevicePixelRatio; }
+        void setUseDevicePixelRatio(bool useDevicePixelRatio);
+
         inline bool decoratePages() const { return m_decoratePages; }
         void setDecoratePages(bool decorate);
 
@@ -82,6 +88,12 @@ public:
         Qt::KeyboardModifiers addAnnotationModifiers() const;
         void setAddAnnotationModifiers(const Qt::KeyboardModifiers& modifiers);
 
+        bool annotationOverlay() const;
+        void setAnnotationOverlay(bool overlay);
+
+        bool formFieldOverlay() const;
+        void setFormFieldOverlay(bool overlay);
+
     private:
         PageItem(QSettings* settings);
         friend class Settings;
@@ -93,6 +105,9 @@ public:
         QIcon m_progressIcon;
         QIcon m_errorIcon;
 
+        bool m_keepObsoletePixmaps;
+        bool m_useDevicePixelRatio;
+
         bool m_decoratePages;
         bool m_decorateLinks;
         bool m_decorateFormFields;
@@ -115,6 +130,9 @@ public:
         int screen() const;
         void setScreen(int screen);
 
+        QColor backgroundColor() const;
+        void setBackgroundColor(const QColor& backgroundColor);
+
     private:
         PresentationView(QSettings* settings);
         friend class Settings;
@@ -233,6 +251,9 @@ public:
         bool trackRecentlyUsed() const;
         void setTrackRecentlyUsed(bool on);
 
+        int recentlyUsedCount() const;
+        void setRecentlyUsedCount(int recentlyUsedCount);
+
         QStringList recentlyUsed() const;
         void setRecentlyUsed(const QStringList& recentlyUsed);
 
@@ -257,6 +278,12 @@ public:
         bool currentPageInWindowTitle() const;
         void setCurrentPageInWindowTitle(bool currentPageInWindowTitle);
 
+        bool instanceNameInWindowTitle() const;
+        void setInstanceNameInWindowTitle(bool instanceNameInWindowTitle);
+
+        bool synchronizeOutlineView() const;
+        void setSynchronizeOutlineView(bool synchronizeOutlineView);
+
         QStringList fileToolBar() const;
         void setFileToolBar(const QStringList& fileToolBar);
 
@@ -284,6 +311,9 @@ public:
         QString savePath() const;
         void setSavePath(const QString& savePath);
 
+        QSize settingsDialogSize(const QSize& sizeHint) const;
+        void setSettingsDialogSize(const QSize& settingsDialogSize);
+
         QSize fontsDialogSize(const QSize& sizeHint) const;
         void setFontsDialogSize(const QSize& fontsDialogSize);
 
@@ -347,6 +377,8 @@ public:
     PrintDialog& printDialog();
 
 private:
+    Q_DISABLE_COPY(Settings)
+
     static Settings* s_instance;
     Settings(QObject* parent = 0);
 
@@ -370,6 +402,9 @@ public:
     public:
         static inline int cacheSize() { return 32 * 1024 * 1024; }
 
+        static inline bool keepObsoletePixmaps() { return false; }
+        static inline bool useDevicePixelRatio() { return false; }
+
         static inline bool decoratePages() { return true; }
         static inline bool decorateLinks() { return true; }
         static inline bool decorateFormFields() { return true; }
@@ -383,6 +418,9 @@ public:
         static inline Qt::KeyboardModifiers copyToClipboardModifiers() { return Qt::ShiftModifier; }
         static inline Qt::KeyboardModifiers addAnnotationModifiers() { return Qt::ControlModifier; }
 
+        static inline bool annotationOverlay() { return false; }
+        static inline bool formFieldOverlay() { return true; }
+
     private:
         PageItem() {}
 
@@ -394,6 +432,8 @@ public:
         static inline bool sync() { return false; }
         static inline int screen() { return -1; }
 
+        static inline QColor backgroundColor() { return QColor(); }
+
     private:
         PresentationView() {}
 
@@ -462,6 +502,8 @@ public:
     public:
         static inline bool trackRecentlyUsed() { return false; }
 
+        static inline int recentlyUsedCount() { return 10; }
+
         static inline bool restoreTabs() { return false; }
         static inline bool restoreBookmarks() { return false; }
         static inline bool restorePerFileSettings() { return false; }
@@ -472,6 +514,9 @@ public:
         static inline bool newTabNextToCurrentTab() { return true; }
 
         static inline bool currentPageInWindowTitle() { return false; }
+        static inline bool instancfeNameInWindowTitle() { return false; }
+
+        static inline bool synchronizeOutlineView() { return false; }
 
         static inline QStringList fileToolBar() { return QStringList() << "openInNewTab" << "refresh"; }
         static inline QStringList editToolBar() { return QStringList() << "currentPage" << "previousPage" << "nextPage"; }
