@@ -28,10 +28,8 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QDialogButtonBox>
 #include <QFormLayout>
 #include <QHeaderView>
-#include <QLineEdit>
 #include <QPushButton>
 #include <QShortcut>
-#include <QSpinBox>
 #include <QTableView>
 
 #include "settings.h"
@@ -41,6 +39,18 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include "documentview.h"
 #include "miscellaneous.h"
 
+static void setCurrentTextToColorName(QComboBox* comboBox, const QColor& color)
+{
+    comboBox->lineEdit()->setText(color.isValid() ? color.name() : QString());
+}
+
+static QColor getValidColorFromCurrentText(QComboBox* comboBox, const QColor& defaultColor)
+{
+    const QColor color(comboBox->currentText());
+
+    return color.isValid() ? color : defaultColor;
+}
+
 Settings* SettingsDialog::s_settings = 0;
 
 SettingsDialog::SettingsDialog(QWidget* parent) : QDialog(parent)
@@ -115,6 +125,9 @@ SettingsDialog::SettingsDialog(QWidget* parent) : QDialog(parent)
     m_defaultsButton = m_dialogButtonBox->addButton(tr("Defaults"), QDialogButtonBox::ResetRole);
     connect(m_defaultsButton, SIGNAL(clicked()), SLOT(reset()));
 
+    m_defaultsOnCurrentTabButton = m_dialogButtonBox->addButton(tr("Defaults on current tab"), QDialogButtonBox::ResetRole);
+    connect(m_defaultsOnCurrentTabButton, SIGNAL(clicked()), SLOT(resetCurrentTab()));
+
     m_behaviorLayout = new QFormLayout(m_tabWidget->widget(0));
     m_interfaceLayout = new QFormLayout(m_tabWidget->widget(2));
     m_modifiersLayout = new QFormLayout(m_tabWidget->widget(4));
@@ -166,27 +179,29 @@ void SettingsDialog::accept()
     s_settings->presentationView().setSync(m_presentationSyncCheckBox->isChecked());
     s_settings->presentationView().setScreen(m_presentationScreenSpinBox->value());
 
-    const QColor highlightColor(m_highlightColorComboBox->currentText());
-    s_settings->pageItem().setHighlightColor(highlightColor.isValid() ? highlightColor : Defaults::PageItem::highlightColor());
-
     s_settings->documentView().setHighlightDuration(m_highlightDurationSpinBox->value());
-
-    const QColor annotationColor(m_annotationColorComboBox->currentText());
-    s_settings->pageItem().setAnnotationColor(annotationColor.isValid() ? annotationColor : Defaults::PageItem::annotationColor());
+    s_settings->pageItem().setHighlightColor(getValidColorFromCurrentText(m_highlightColorComboBox, Defaults::PageItem::highlightColor()));
+    s_settings->pageItem().setAnnotationColor(getValidColorFromCurrentText(m_annotationColorComboBox, Defaults::PageItem::annotationColor()));
 
     s_settings->documentView().setSourceEditor(m_sourceEditorLineEdit->text());
 
     // graphics
 
+    s_settings->pageItem().setKeepObsoletePixmaps(m_keepObsoletePixmapsCheckBox->isChecked());
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+
+    s_settings->pageItem().setUseDevicePixelRatio(m_useDevicePixelRatioCheckBox->isChecked());
+
+#endif // QT_VERSION
+
     s_settings->pageItem().setDecoratePages(m_decoratePagesCheckBox->isChecked());
     s_settings->pageItem().setDecorateLinks(m_decorateLinksCheckBox->isChecked());
     s_settings->pageItem().setDecorateFormFields(m_decorateFormFieldsCheckBox->isChecked());
 
-    const QColor backgroundColor(m_backgroundColorComboBox->currentText());
-    s_settings->pageItem().setBackgroundColor(backgroundColor.isValid() ? backgroundColor : Defaults::PageItem::backgroundColor());
-
-    const QColor paperColor(m_paperColorComboBox->currentText());
-    s_settings->pageItem().setPaperColor(paperColor.isValid() ? paperColor : Defaults::PageItem::paperColor());
+    s_settings->pageItem().setBackgroundColor(getValidColorFromCurrentText(m_backgroundColorComboBox, Defaults::PageItem::backgroundColor()));
+    s_settings->pageItem().setPaperColor(getValidColorFromCurrentText(m_paperColorComboBox, Defaults::PageItem::paperColor()));
+    s_settings->presentationView().setBackgroundColor(getValidColorFromCurrentText(m_presentationBackgroundColorComboBox, Defaults::PresentationView::backgroundColor()));
 
     s_settings->documentView().setPagesPerRow(m_pagesPerRowSpinBox->value());
 
@@ -206,15 +221,23 @@ void SettingsDialog::accept()
 
     s_settings->mainWindow().setNewTabNextToCurrentTab(m_newTabNextToCurrentTabCheckBox->isChecked());
 
+    s_settings->mainWindow().setRecentlyUsedCount(m_recentlyUsedCountSpinBox->value());
+
     s_settings->mainWindow().setFileToolBar(m_fileToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
     s_settings->mainWindow().setEditToolBar(m_editToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
     s_settings->mainWindow().setViewToolBar(m_viewToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
 
     s_settings->mainWindow().setCurrentPageInWindowTitle(m_currentPageInWindowTitleCheckBox->isChecked());
+    s_settings->mainWindow().setInstanceNameInWindowTitle(m_instanceNameInWindowTitleCheckBox->isChecked());
+
+    s_settings->mainWindow().setSynchronizeOutlineView(m_synchronizeOutlineViewCheckBox->isChecked());
 
     s_settings->documentView().setHighlightCurrentThumbnail(m_highlightCurrentThumbnailCheckBox->isChecked());
     s_settings->documentView().setLimitThumbnailsToResults(m_limitThumbnailsToResultsCheckBox->isChecked());
 
+    s_settings->pageItem().setAnnotationOverlay(m_annotationOverlayCheckBox->isChecked());
+    s_settings->pageItem().setFormFieldOverlay(m_formFieldOverlayCheckBox);
+
     // modifiers
 
     s_settings->documentView().setZoomModifiers(static_cast<Qt::KeyboardModifier>(m_zoomModifiersComboBox->itemData(m_zoomModifiersComboBox->currentIndex()).toInt()));
@@ -227,91 +250,40 @@ void SettingsDialog::accept()
 
 void SettingsDialog::reset()
 {
-#ifdef WITH_PDF
+    resetBehaviorTab();
 
-    if(m_pdfSettingsWidget != 0)
-    {
-        m_pdfSettingsWidget->reset();
-    }
+    resetGraphicsTab();
 
-#endif // WITH_PDF
-
-#ifdef WITH_PS
-
-    if(m_psSettingsWidget != 0)
-    {
-        m_psSettingsWidget->reset();
-    }
-
-#endif // WITH_PS
+    resetInterfaceTab();
 
     ShortcutHandler::instance()->reset();
 
-    // behavior
+    resetModifiersTab();
+}
 
-    m_openUrlCheckBox->setChecked(Defaults::DocumentView::openUrl());
-
-    m_autoRefreshCheckBox->setChecked(Defaults::DocumentView::autoRefresh());
-
-    m_trackRecentlyUsedCheckBox->setChecked(Defaults::MainWindow::trackRecentlyUsed());
-
-    m_restoreTabsCheckBox->setChecked(Defaults::MainWindow::restoreTabs());
-    m_restoreBookmarksCheckBox->setChecked(Defaults::MainWindow::restoreBookmarks());
-    m_restorePerFileSettingsCheckBox->setChecked(Defaults::MainWindow::restorePerFileSettings());
-
-    m_presentationSyncCheckBox->setChecked(Defaults::PresentationView::sync());
-    m_presentationScreenSpinBox->setValue(Defaults::PresentationView::screen());
-
-    m_highlightColorComboBox->lineEdit()->setText(Defaults::PageItem::highlightColor().name());
-    m_highlightDurationSpinBox->setValue(Defaults::DocumentView::highlightDuration());
-    m_annotationColorComboBox->lineEdit()->setText(Defaults::PageItem::annotationColor().name());
-
-    m_sourceEditorLineEdit->clear();
-
-    // graphics
-
-    m_decoratePagesCheckBox->setChecked(Defaults::PageItem::decoratePages());
-    m_decorateLinksCheckBox->setChecked(Defaults::PageItem::decorateLinks());
-    m_decorateFormFieldsCheckBox->setChecked(Defaults::PageItem::decorateFormFields());
-
-    m_backgroundColorComboBox->lineEdit()->setText(Defaults::PageItem::backgroundColor().name());
-    m_paperColorComboBox->lineEdit()->setText(Defaults::PageItem::paperColor().name());
-
-    m_pagesPerRowSpinBox->setValue(Defaults::DocumentView::pagesPerRow());
-
-    m_pageSpacingSpinBox->setValue(Defaults::DocumentView::pageSpacing());
-    m_thumbnailSpacingSpinBox->setValue(Defaults::DocumentView::thumbnailSpacing());
-
-    m_thumbnailSizeSpinBox->setValue(Defaults::DocumentView::thumbnailSize());
-
-    m_cacheSizeComboBox->setCurrentIndex(m_cacheSizeComboBox->findData(Defaults::PageItem::cacheSize()));
-    m_prefetchCheckBox->setChecked(Defaults::DocumentView::prefetch());
-    m_prefetchDistanceSpinBox->setValue(Defaults::DocumentView::prefetchDistance());
-
-    // interface
-
-    m_tabPositionComboBox->setCurrentIndex(m_tabPositionComboBox->findData(static_cast< uint >(Defaults::MainWindow::tabPosition())));
-    m_tabVisibilityComboBox->setCurrentIndex(m_tabVisibilityComboBox->findData(static_cast< uint >(Defaults::MainWindow::tabVisibility())));
-
-    m_newTabNextToCurrentTabCheckBox->setChecked(Defaults::MainWindow::newTabNextToCurrentTab());
-
-    m_fileToolBarLineEdit->setText(Defaults::MainWindow::fileToolBar().join(","));
-    m_editToolBarLineEdit->setText(Defaults::MainWindow::editToolBar().join(","));
-    m_viewToolBarLineEdit->setText(Defaults::MainWindow::viewToolBar().join(","));
-
-    m_currentPageInWindowTitleCheckBox->setChecked(Defaults::MainWindow::currentPageInWindowTitle());
-
-    m_highlightCurrentThumbnailCheckBox->setChecked(Defaults::DocumentView::highlightCurrentThumbnail());
-    m_limitThumbnailsToResultsCheckBox->setChecked(Defaults::DocumentView::limitThumbnailsToResults());
-
-    // modifiers
-
-    m_zoomModifiersComboBox->setCurrentIndex(m_zoomModifiersComboBox->findData(static_cast< int >(Defaults::DocumentView::zoomModifiers())));
-    m_rotateModifiersComboBox->setCurrentIndex(m_rotateModifiersComboBox->findData(static_cast< int >(Defaults::DocumentView::rotateModifiers())));
-    m_scrollModifiersComboBox->setCurrentIndex(m_scrollModifiersComboBox->findData(static_cast< int >(Defaults::DocumentView::scrollModifiers())));
-
-    m_copyToClipboardModifiersComboBox->setCurrentIndex(m_copyToClipboardModifiersComboBox->findData(static_cast< int >(Defaults::PageItem::copyToClipboardModifiers())));
-    m_addAnnotationModifiersComboBox->setCurrentIndex(m_addAnnotationModifiersComboBox->findData(static_cast< int >(Defaults::PageItem::addAnnotationModifiers())));
+void SettingsDialog::resetCurrentTab()
+{
+    switch(m_tabWidget->currentIndex())
+    {
+    default:
+        reset();
+        break;
+    case 0:
+        resetBehaviorTab();
+        break;
+    case 1:
+        resetGraphicsTab();
+        break;
+    case 2:
+        resetInterfaceTab();
+        break;
+    case 3:
+        ShortcutHandler::instance()->reset();
+        break;
+    case 4:
+        resetModifiersTab();
+        break;
+    }
 }
 
 void SettingsDialog::createBehaviorTab()
@@ -383,16 +355,6 @@ void SettingsDialog::createBehaviorTab()
 
     m_behaviorLayout->addRow(tr("Presentation screen:"), m_presentationScreenSpinBox);
 
-    // highlight color
-
-    m_highlightColorComboBox = new QComboBox(this);
-    m_highlightColorComboBox->setEditable(true);
-    m_highlightColorComboBox->setInsertPolicy(QComboBox::NoInsert);
-    m_highlightColorComboBox->addItems(QColor::colorNames());
-    m_highlightColorComboBox->lineEdit()->setText(s_settings->pageItem().highlightColor().name());
-
-    m_behaviorLayout->addRow(tr("Highlight color:"), m_highlightColorComboBox);
-
     // highlight duration
 
     m_highlightDurationSpinBox = new QSpinBox(this);
@@ -404,13 +366,15 @@ void SettingsDialog::createBehaviorTab()
 
     m_behaviorLayout->addRow(tr("Highlight duration:"), m_highlightDurationSpinBox);
 
+    // highlight color
+
+    createColorComboBox(m_highlightColorComboBox, s_settings->pageItem().highlightColor());
+
+    m_behaviorLayout->addRow(tr("Highlight color:"), m_highlightColorComboBox);
+
     // annotation color
 
-    m_annotationColorComboBox = new QComboBox(this);
-    m_annotationColorComboBox->setEditable(true);
-    m_annotationColorComboBox->setInsertPolicy(QComboBox::NoInsert);
-    m_annotationColorComboBox->addItems(QColor::colorNames());
-    m_annotationColorComboBox->lineEdit()->setText(s_settings->pageItem().annotationColor().name());
+    createColorComboBox(m_annotationColorComboBox, s_settings->pageItem().annotationColor());
 
     m_behaviorLayout->addRow(tr("Annotation color:"), m_annotationColorComboBox);
 
@@ -423,8 +387,48 @@ void SettingsDialog::createBehaviorTab()
     m_behaviorLayout->addRow(tr("Source editor:"), m_sourceEditorLineEdit);
 }
 
+void SettingsDialog::resetBehaviorTab()
+{
+    m_openUrlCheckBox->setChecked(Defaults::DocumentView::openUrl());
+
+    m_autoRefreshCheckBox->setChecked(Defaults::DocumentView::autoRefresh());
+
+    m_trackRecentlyUsedCheckBox->setChecked(Defaults::MainWindow::trackRecentlyUsed());
+
+    m_restoreTabsCheckBox->setChecked(Defaults::MainWindow::restoreTabs());
+    m_restoreBookmarksCheckBox->setChecked(Defaults::MainWindow::restoreBookmarks());
+    m_restorePerFileSettingsCheckBox->setChecked(Defaults::MainWindow::restorePerFileSettings());
+
+    m_presentationSyncCheckBox->setChecked(Defaults::PresentationView::sync());
+    m_presentationScreenSpinBox->setValue(Defaults::PresentationView::screen());
+
+    m_highlightDurationSpinBox->setValue(Defaults::DocumentView::highlightDuration());
+    setCurrentTextToColorName(m_highlightColorComboBox, Defaults::PageItem::highlightColor());
+    setCurrentTextToColorName(m_annotationColorComboBox, Defaults::PageItem::annotationColor());
+
+    m_sourceEditorLineEdit->clear();
+}
+
 void SettingsDialog::createGraphicsTab()
 {
+    // keep obsolete pixmaps
+
+    m_keepObsoletePixmapsCheckBox = new QCheckBox(this);
+    m_keepObsoletePixmapsCheckBox->setChecked(s_settings->pageItem().keepObsoletePixmaps());
+
+    m_graphicsLayout->addRow(tr("Keep obsolete pixmaps:"), m_keepObsoletePixmapsCheckBox);
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+
+    // use device pixel ratio
+
+    m_useDevicePixelRatioCheckBox = new QCheckBox(this);
+    m_useDevicePixelRatioCheckBox->setChecked(s_settings->pageItem().useDevicePixelRatio());
+
+    m_graphicsLayout->addRow(tr("Use device pixel ratio:"), m_useDevicePixelRatioCheckBox);
+
+#endif // QT_VERSION
+
     // decorate pages
 
     m_decoratePagesCheckBox = new QCheckBox(this);
@@ -448,24 +452,22 @@ void SettingsDialog::createGraphicsTab()
 
     // background color
 
-    m_backgroundColorComboBox = new QComboBox(this);
-    m_backgroundColorComboBox->setEditable(true);
-    m_backgroundColorComboBox->setInsertPolicy(QComboBox::NoInsert);
-    m_backgroundColorComboBox->addItems(QColor::colorNames());
-    m_backgroundColorComboBox->lineEdit()->setText(s_settings->pageItem().backgroundColor().name());
+    createColorComboBox(m_backgroundColorComboBox, s_settings->pageItem().backgroundColor());
 
     m_graphicsLayout->addRow(tr("Background color:"), m_backgroundColorComboBox);
 
     // paper color
 
-    m_paperColorComboBox = new QComboBox(this);
-    m_paperColorComboBox->setEditable(true);
-    m_paperColorComboBox->setInsertPolicy(QComboBox::NoInsert);
-    m_paperColorComboBox->addItems(QColor::colorNames());
-    m_paperColorComboBox->lineEdit()->setText(s_settings->pageItem().paperColor().name());
+    createColorComboBox(m_paperColorComboBox, s_settings->pageItem().paperColor());
 
     m_graphicsLayout->addRow(tr("Paper color:"), m_paperColorComboBox);
 
+    // presentation background color
+
+    createColorComboBox(m_presentationBackgroundColorComboBox, s_settings->presentationView().backgroundColor());
+
+    m_graphicsLayout->addRow(tr("Presentation background color:"), m_presentationBackgroundColorComboBox);
+
     // pages per row
 
     m_pagesPerRowSpinBox = new QSpinBox(this);
@@ -535,6 +537,54 @@ void SettingsDialog::createGraphicsTab()
     m_graphicsLayout->addRow(tr("Prefetch distance:"), m_prefetchDistanceSpinBox);
 }
 
+void SettingsDialog::resetGraphicsTab()
+{
+#ifdef WITH_PDF
+
+    if(m_pdfSettingsWidget != 0)
+    {
+        m_pdfSettingsWidget->reset();
+    }
+
+#endif // WITH_PDF
+
+#ifdef WITH_PS
+
+    if(m_psSettingsWidget != 0)
+    {
+        m_psSettingsWidget->reset();
+    }
+
+#endif // WITH_PS
+
+    m_keepObsoletePixmapsCheckBox->setChecked(Defaults::PageItem::keepObsoletePixmaps());
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+
+    m_useDevicePixelRatioCheckBox->setChecked(Defaults::PageItem::useDevicePixelRatio());
+
+#endif // QT_VERSION
+
+    m_decoratePagesCheckBox->setChecked(Defaults::PageItem::decoratePages());
+    m_decorateLinksCheckBox->setChecked(Defaults::PageItem::decorateLinks());
+    m_decorateFormFieldsCheckBox->setChecked(Defaults::PageItem::decorateFormFields());
+
+    setCurrentTextToColorName(m_backgroundColorComboBox, Defaults::PageItem::backgroundColor());
+    setCurrentTextToColorName(m_paperColorComboBox, Defaults::PageItem::paperColor());
+    setCurrentTextToColorName(m_presentationBackgroundColorComboBox, Defaults::PresentationView::backgroundColor());
+
+    m_pagesPerRowSpinBox->setValue(Defaults::DocumentView::pagesPerRow());
+
+    m_pageSpacingSpinBox->setValue(Defaults::DocumentView::pageSpacing());
+    m_thumbnailSpacingSpinBox->setValue(Defaults::DocumentView::thumbnailSpacing());
+
+    m_thumbnailSizeSpinBox->setValue(Defaults::DocumentView::thumbnailSize());
+
+    m_cacheSizeComboBox->setCurrentIndex(m_cacheSizeComboBox->findData(Defaults::PageItem::cacheSize()));
+    m_prefetchCheckBox->setChecked(Defaults::DocumentView::prefetch());
+    m_prefetchDistanceSpinBox->setValue(Defaults::DocumentView::prefetchDistance());
+}
+
 void SettingsDialog::createInterfaceTab()
 {
     // tab position
@@ -565,6 +615,15 @@ void SettingsDialog::createInterfaceTab()
 
     m_interfaceLayout->addRow(tr("New tab next to current tab:"), m_newTabNextToCurrentTabCheckBox);
 
+    // recently used count
+
+    m_recentlyUsedCountSpinBox = new QSpinBox(this);
+    m_recentlyUsedCountSpinBox->setRange(1, 50);
+    m_recentlyUsedCountSpinBox->setValue(s_settings->mainWindow().recentlyUsedCount());
+    m_recentlyUsedCountSpinBox->setToolTip(tr("Effective after restart."));
+
+    m_interfaceLayout->addRow(tr("Recently used count:"), m_recentlyUsedCountSpinBox);
+
     // file tool bar
 
     m_fileToolBarLineEdit = new QLineEdit(this);
@@ -596,6 +655,20 @@ void SettingsDialog::createInterfaceTab()
 
     m_interfaceLayout->addRow(tr("Current page in window title:"), m_currentPageInWindowTitleCheckBox);
 
+    // instance name in window title
+
+    m_instanceNameInWindowTitleCheckBox = new QCheckBox(this);
+    m_instanceNameInWindowTitleCheckBox->setChecked(s_settings->mainWindow().instanceNameInWindowTitle());
+
+    m_interfaceLayout->addRow(tr("Instance name in window title:"), m_instanceNameInWindowTitleCheckBox);
+
+    // synchronize outline view
+
+    m_synchronizeOutlineViewCheckBox = new QCheckBox(this);
+    m_synchronizeOutlineViewCheckBox->setChecked(s_settings->mainWindow().synchronizeOutlineView());
+
+    m_interfaceLayout->addRow(tr("Synchronize outline view:"), m_synchronizeOutlineViewCheckBox);
+
     // highlight current thumbnail
 
     m_highlightCurrentThumbnailCheckBox = new QCheckBox(this);
@@ -609,6 +682,45 @@ void SettingsDialog::createInterfaceTab()
     m_limitThumbnailsToResultsCheckBox->setChecked(s_settings->documentView().limitThumbnailsToResults());
 
     m_interfaceLayout->addRow(tr("Limit thumbnails to results:"), m_limitThumbnailsToResultsCheckBox);
+
+    // annotation overlay
+
+    m_annotationOverlayCheckBox = new QCheckBox(this);
+    m_annotationOverlayCheckBox->setChecked(s_settings->pageItem().annotationOverlay());
+
+    m_interfaceLayout->addRow(tr("Annotation overlay:"), m_annotationOverlayCheckBox);
+
+    // form field overlay
+
+    m_formFieldOverlayCheckBox = new QCheckBox(this);
+    m_formFieldOverlayCheckBox->setChecked(s_settings->pageItem().formFieldOverlay());
+
+    m_interfaceLayout->addRow(tr("Form field overlay:"), m_formFieldOverlayCheckBox);
+}
+
+void SettingsDialog::resetInterfaceTab()
+{
+    m_tabPositionComboBox->setCurrentIndex(m_tabPositionComboBox->findData(static_cast< uint >(Defaults::MainWindow::tabPosition())));
+    m_tabVisibilityComboBox->setCurrentIndex(m_tabVisibilityComboBox->findData(static_cast< uint >(Defaults::MainWindow::tabVisibility())));
+
+    m_newTabNextToCurrentTabCheckBox->setChecked(Defaults::MainWindow::newTabNextToCurrentTab());
+
+    m_recentlyUsedCountSpinBox->setValue(Defaults::MainWindow::recentlyUsedCount());
+
+    m_fileToolBarLineEdit->setText(Defaults::MainWindow::fileToolBar().join(","));
+    m_editToolBarLineEdit->setText(Defaults::MainWindow::editToolBar().join(","));
+    m_viewToolBarLineEdit->setText(Defaults::MainWindow::viewToolBar().join(","));
+
+    m_currentPageInWindowTitleCheckBox->setChecked(Defaults::MainWindow::currentPageInWindowTitle());
+    m_instanceNameInWindowTitleCheckBox->setChecked(Defaults::MainWindow::instancfeNameInWindowTitle());
+
+    m_synchronizeOutlineViewCheckBox->setChecked(Defaults::MainWindow::synchronizeOutlineView());
+
+    m_highlightCurrentThumbnailCheckBox->setChecked(Defaults::DocumentView::highlightCurrentThumbnail());
+    m_limitThumbnailsToResultsCheckBox->setChecked(Defaults::DocumentView::limitThumbnailsToResults());
+
+    m_annotationOverlayCheckBox->setChecked(Defaults::PageItem::annotationOverlay());
+    m_formFieldOverlayCheckBox->setChecked(Defaults::PageItem::formFieldOverlay());
 }
 
 void SettingsDialog::createModifiersTab()
@@ -644,6 +756,26 @@ void SettingsDialog::createModifiersTab()
     m_modifiersLayout->addRow(tr("Add annotation:"), m_addAnnotationModifiersComboBox);
 }
 
+void SettingsDialog::resetModifiersTab()
+{
+    m_zoomModifiersComboBox->setCurrentIndex(m_zoomModifiersComboBox->findData(static_cast< int >(Defaults::DocumentView::zoomModifiers())));
+    m_rotateModifiersComboBox->setCurrentIndex(m_rotateModifiersComboBox->findData(static_cast< int >(Defaults::DocumentView::rotateModifiers())));
+    m_scrollModifiersComboBox->setCurrentIndex(m_scrollModifiersComboBox->findData(static_cast< int >(Defaults::DocumentView::scrollModifiers())));
+
+    m_copyToClipboardModifiersComboBox->setCurrentIndex(m_copyToClipboardModifiersComboBox->findData(static_cast< int >(Defaults::PageItem::copyToClipboardModifiers())));
+    m_addAnnotationModifiersComboBox->setCurrentIndex(m_addAnnotationModifiersComboBox->findData(static_cast< int >(Defaults::PageItem::addAnnotationModifiers())));
+}
+
+void SettingsDialog::createColorComboBox(QComboBox*& comboBox, const QColor& color)
+{
+    comboBox = new QComboBox(this);
+    comboBox->setEditable(true);
+    comboBox->setInsertPolicy(QComboBox::NoInsert);
+    comboBox->addItems(QColor::colorNames());
+
+    setCurrentTextToColorName(comboBox, color);
+}
+
 void SettingsDialog::createModifiersComboBox(QComboBox*& comboBox, const Qt::KeyboardModifiers& modifiers)
 {
     comboBox = new QComboBox(this);
