@@ -48,8 +48,11 @@ public:
 public slots:
     void accept();
     void reset();
+    void resetCurrentTab();
 
 private:
+    Q_DISABLE_COPY(SettingsDialog)
+
     static Settings* s_settings;
 
     QTabWidget* m_graphicsTabWidget;
@@ -72,6 +75,7 @@ private:
     QTabWidget* m_tabWidget;
     QDialogButtonBox* m_dialogButtonBox;
     QPushButton* m_defaultsButton;
+    QPushButton* m_defaultsOnCurrentTabButton;
 
     QFormLayout* m_behaviorLayout;
     QFormLayout* m_interfaceLayout;
@@ -92,22 +96,32 @@ private:
     QCheckBox* m_presentationSyncCheckBox;
     QSpinBox* m_presentationScreenSpinBox;
 
-    QComboBox* m_highlightColorComboBox;
     QSpinBox* m_highlightDurationSpinBox;
+    QComboBox* m_highlightColorComboBox;
     QComboBox* m_annotationColorComboBox;
 
     QLineEdit* m_sourceEditorLineEdit;
 
     void createBehaviorTab();
+    void resetBehaviorTab();
 
     // graphics
 
+    QCheckBox* m_keepObsoletePixmapsCheckBox;
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+
+    QCheckBox* m_useDevicePixelRatioCheckBox;
+
+#endif // QT_VERSION
+
     QCheckBox* m_decoratePagesCheckBox;
     QCheckBox* m_decorateLinksCheckBox;
     QCheckBox* m_decorateFormFieldsCheckBox;
 
     QComboBox* m_backgroundColorComboBox;
     QComboBox* m_paperColorComboBox;
+    QComboBox* m_presentationBackgroundColorComboBox;
 
     QSpinBox* m_pagesPerRowSpinBox;
 
@@ -121,6 +135,7 @@ private:
     QSpinBox* m_prefetchDistanceSpinBox;
 
     void createGraphicsTab();
+    void resetGraphicsTab();
 
     // interface
 
@@ -129,16 +144,25 @@ private:
 
     QCheckBox* m_newTabNextToCurrentTabCheckBox;
 
+    QSpinBox* m_recentlyUsedCountSpinBox;
+
     QLineEdit* m_fileToolBarLineEdit;
     QLineEdit* m_editToolBarLineEdit;
     QLineEdit* m_viewToolBarLineEdit;
 
     QCheckBox* m_currentPageInWindowTitleCheckBox;
+    QCheckBox* m_instanceNameInWindowTitleCheckBox;
+
+    QCheckBox* m_synchronizeOutlineViewCheckBox;
 
     QCheckBox* m_highlightCurrentThumbnailCheckBox;
     QCheckBox* m_limitThumbnailsToResultsCheckBox;
 
+    QCheckBox* m_annotationOverlayCheckBox;
+    QCheckBox* m_formFieldOverlayCheckBox;
+
     void createInterfaceTab();
+    void resetInterfaceTab();
 
     // modifiers
 
@@ -150,6 +174,11 @@ private:
     QComboBox* m_addAnnotationModifiersComboBox;
 
     void createModifiersTab();
+    void resetModifiersTab();
+
+    // helper methods
+
+    void createColorComboBox(QComboBox*& comboBox, const QColor& color);
     void createModifiersComboBox(QComboBox*& comboBox, const Qt::KeyboardModifiers& modifiers);
 
 };
